@@ -137,8 +137,8 @@ extern int	errno;
 */
 #define GKRELLMD_VERSION_MAJOR   2
 #define GKRELLMD_VERSION_MINOR   1
-#define GKRELLMD_VERSION_REV     7
-#define GKRELLMD_EXTRAVERSION    "a"
+#define GKRELLMD_VERSION_REV     8
+#define GKRELLMD_EXTRAVERSION    ""
 
 #define GKRELLMD_CONFIG     "gkrellmd.conf"
 
@@ -202,11 +202,14 @@ struct GkrellmdConfig
 	gint		*server_fd;
 	gint		max_clients;
 	gint		server_port;
-	gboolean	verbose;
+	gint		verbose;
 	time_t		start_time;
+	gint		io_timeout;
+	gint		reconnect_timeout;
 
 	gint		fs_interval,
-				nfs_interval;
+				nfs_interval,
+				inet_interval;
 	};
 
 extern struct GkrellmdConfig	_GK;
