@@ -292,10 +292,16 @@ parse_config(gchar *config, gchar *arg)
 		_GK.max_clients = atoi(arg);
 	else if (!strcmp(config, "allow-host") || !strcmp(config, "a"))
 		allow_host_list = g_list_append(allow_host_list, g_strdup(arg));
+	else if (!strcmp(config, "io-timeout"))
+		_GK.io_timeout = atoi(arg);
+	else if (!strcmp(config, "reconnect-timeout"))
+		_GK.reconnect_timeout = atoi(arg);
 	else if (!strcmp(config, "fs-interval"))
 		_GK.fs_interval = atoi(arg);
 	else if (!strcmp(config, "nfs-interval"))
 		_GK.nfs_interval = atoi(arg);
+	else if (!strcmp(config, "inet-interval"))
+		_GK.inet_interval = atoi(arg);
 #if !defined(WIN32)
 	else if (!strcmp(config, "user") || !strcmp(config, "U"))
 		{
@@ -352,6 +358,7 @@ read_config(void)
 
 	_GK.fs_interval = 2;
 	_GK.nfs_interval = 16;
+	_GK.inet_interval = 1;
 
 #if defined(GKRELLMD_SYS_ETC)
 	path = g_strdup_printf("%s/%s", GKRELLMD_SYS_ETC, GKRELLMD_CONFIG);
@@ -414,7 +421,7 @@ get_args(gint argc, gchar **argv)
 			}
 		if (!strcmp(opt, "verbose") || !strcmp(opt, "V"))
 			{
-			_GK.verbose = TRUE;
+			_GK.verbose += 1;
 			continue;
 			}
 		else if (!strcmp(opt, "help") || !strcmp(opt, "h"))
@@ -589,7 +596,7 @@ detach_from_terminal(void)
 	chdir("/");
 #endif
 
-	signal(SIGCHLD, SIG_IGN);
+//	signal(SIGCHLD, SIG_IGN);
 	signal(SIGTSTP, SIG_IGN);
 	signal(SIGTTOU, SIG_IGN);
 	signal(SIGTTIN, SIG_IGN);
@@ -661,6 +668,8 @@ main(gint argc, gchar **argv)
 	read_config();
 	get_args(argc, argv);
 
+	if (_GK.verbose)
+		printf("update_HZ=%d\n", _GK.update_HZ);
 	if (detach_flag)
 		detach_from_terminal();
 	else
@@ -682,6 +691,8 @@ main(gint argc, gchar **argv)
 		_GK.fs_interval = 2;
 	if (_GK.nfs_interval < 1 || _GK.nfs_interval > 10000)
 		_GK.nfs_interval = 16;
+	if (_GK.inet_interval > 20)
+		_GK.inet_interval = 20;
 
 	load_monitors();
 
@@ -695,17 +706,17 @@ main(gint argc, gchar **argv)
 	/* Listen on the socket so a client gkrellm can connect.
 	*/
 	FD_ZERO(&read_fds);
-	for (i = 1; i <= _GK.server_fd[0]; ++i) {
+	for (i = 1; i <= _GK.server_fd[0]; ++i)
+		{
 		if (listen(_GK.server_fd[i], 5) == -1)
 			{
-			fprintf(stderr, "gkrellmd listen() failed: %s\n",
-				strerror(errno));
+			fprintf(stderr, "gkrellmd listen() failed: %s\n", strerror(errno));
 			exit(1);
 			}
 		FD_SET(_GK.server_fd[i], &read_fds);
 		if (max_fd < _GK.server_fd[i])
 			max_fd = _GK.server_fd[i];
-	}
+		}
 
 	interval = 1000000 / _GK.update_HZ;
 
@@ -770,7 +781,7 @@ main(gint argc, gchar **argv)
 				if (client_fd > max_fd)
 					max_fd = client_fd;
 				client = accept_client(client_fd,
-						       (struct sockaddr *)&client_addr, addr_len);
+							(struct sockaddr *)&client_addr, addr_len);
 				if (!client)
 					{
 					close(client_fd);
@@ -780,8 +791,8 @@ main(gint argc, gchar **argv)
 				gkrellmd_serve_setup(client);
 				if (_GK.verbose)
 					printf("gkrellmd accepted client: %s:%u\n",
-					       client->hostname,
-					       ntohs(((struct sockaddr_in *)&client_addr)->sin_port));
+						client->hostname,
+						ntohs(((struct sockaddr_in *)&client_addr)->sin_port));
 				}
 			else
 				{
