@@ -626,35 +626,40 @@ update_inet(gboolean force)
 	{
 	GList		*list;
 	InetData	*in;
+	static gint	check_tcp;
 
+	
 	if (!force && !GK.second_tick)
 		return;
 
-	free_glist_and_data(&inet_dead_list);
-	inet_new = FALSE;
-
-	for (list = inet_list; list; list = list->next)
+	if (force || check_tcp == 0)
 		{
-		in = (InetData *) list->data;
-		in->alive = FALSE;
-		in->new_connection = FALSE;
-		}
-
-	gkrellm_sys_inet_read_tcp_data();
-
-	for (list = inet_list; list; )
-		{
-		in = (InetData *) list->data;
-		if (!in->alive)
+		free_glist_and_data(&inet_dead_list);
+		inet_new = FALSE;
+		for (list = inet_list; list; list = list->next)
 			{
-			if (list == inet_list)
-				inet_list = inet_list->next;
-			list = g_list_remove(list, in);
-			inet_dead_list = g_list_append(inet_dead_list, in);
+			in = (InetData *) list->data;
+			in->alive = FALSE;
+			in->new_connection = FALSE;
+			}
+
+		gkrellm_sys_inet_read_tcp_data();
+
+		for (list = inet_list; list; )
+			{
+			in = (InetData *) list->data;
+			if (!in->alive)
+				{
+				if (list == inet_list)
+					inet_list = inet_list->next;
+				list = g_list_remove(list, in);
+				inet_dead_list = g_list_append(inet_dead_list, in);
+				}
+			else
+				list = list->next;
 			}
-		else
-			list = list->next;
 		}
+	check_tcp = (check_tcp + 1) % _GK.inet_interval;
 	}
 
 static gchar *
@@ -787,7 +792,7 @@ static GkrellmdMonitor inet_monitor =
 static GkrellmdMonitor *
 init_inet_monitor(void)
 	{
-	if (gkrellm_sys_inet_init())
+	if (_GK.inet_interval > 0 && gkrellm_sys_inet_init())
 		return &inet_monitor;
 	inet_unsupported = TRUE;
 	return NULL;
@@ -2009,6 +2014,16 @@ gkrellmd_serve_setup(GkrellmdClient *client)
 		send_to_client(client->fd, buf);
 		}
 
+	sprintf(buf, "%d\n", _GK.io_timeout);
+	s = g_strconcat("<io_timeout>\n", buf, "\n", NULL);
+	send_to_client(client->fd, s);
+	g_free(s);
+
+	sprintf(buf, "%d\n", _GK.reconnect_timeout);
+	s = g_strconcat("<reconnect_timeout>\n", buf, "\n", NULL);
+	send_to_client(client->fd, s);
+	g_free(s);
+
 	send_to_client(client->fd, "</gkrellmd_setup>\n");
 	}
 
