@@ -23,6 +23,7 @@
 #include "gkrellm.h"
 #include "gkrellm-private.h"
 
+#include <math.h>
 
 static GList	*gkrellm_alert_list;
 static gint		voice_fifo = -1;
@@ -44,19 +45,33 @@ gvoice_fifo_open(void)
 	}
 	
 static void
-send_gvoice(gchar *str)
+send_gvoice(GkrellmAlert *alert, gchar *string)
 	{
-	if (!str || !*str || !gvoice_fifo_open())
+	gchar	str[1024];
+
+	if (!string || !*string || !gvoice_fifo_open())
 		return;
+	if (alert->cb_command_process)
+		(*alert->cb_command_process)(alert, string, str, sizeof(str),
+				alert->cb_command_process_data);
+	else
+		snprintf(str, sizeof(str), "%s", string);
 	write(voice_fifo, str, strlen(str));
 	}
 
 static void
-run_command(gchar *command)
+run_command(GkrellmAlert *alert, gchar *command)
 	{
+	gchar	cmd[1024];
+
 	if (!command || !*command)
 		return;
-	g_spawn_command_line_async(command, NULL);
+	if (alert->cb_command_process)
+		(*alert->cb_command_process)(alert, command, cmd, sizeof(cmd),
+				alert->cb_command_process_data);
+	else
+		snprintf(cmd, sizeof(cmd), "%s", command);
+	g_spawn_command_line_async(cmd, NULL);
 	}
 
 void
@@ -80,6 +95,14 @@ gkrellm_render_default_alert_decal(GkrellmAlert *alert)
 				ad->h * ad->nframes);
 	}
 
+gboolean
+gkrellm_alert_decal_visible(GkrellmAlert *alert)
+	{
+	if (alert && alert->ad.decal)
+		return TRUE;
+	return FALSE;
+	}
+
 static gboolean
 create_alert_objects(GkrellmAlert *alert)
 	{
@@ -151,10 +174,10 @@ trigger_warn(GkrellmAlert *alert, GkrellmTrigger *trigger)
 		create_alert_objects(alert);
 		gkrellm_alert_list = g_list_append(gkrellm_alert_list, alert);
 		if (!alert->suppress_command)
-			run_command(alert->warn_command);
+			run_command(alert, alert->warn_command);
 		alert->suppress_command = FALSE;
 		alert->warn_repeat = alert->warn_repeat_set;
-		send_gvoice(alert->voice_string);
+		send_gvoice(alert, alert->voice_string);
 		alert->voice_repeat = alert->voice_repeat_set;
 		}
 	}
@@ -178,13 +201,14 @@ trigger_alarm(GkrellmAlert *alert, GkrellmTrigger *trigger)
 	if (!trigger->alarm_on)
 		{
 		trigger->alarm_on = TRUE;
+		trigger->warn_delay = 0;
 		create_alert_objects(alert);
 		gkrellm_alert_list = g_list_append(gkrellm_alert_list, alert);
 		if (!alert->suppress_command)
-			run_command(alert->alarm_command);
+			run_command(alert, alert->alarm_command);
 		alert->suppress_command = FALSE;
 		alert->alarm_repeat = alert->alarm_repeat_set;
-		send_gvoice(alert->voice_string);
+		send_gvoice(alert, alert->voice_string);
 		alert->voice_repeat = alert->voice_repeat_set;
 		}
 	}
@@ -216,14 +240,89 @@ gkrellm_thaw_alert(GkrellmAlert *alert)
 		alert->freeze = FALSE;
 	}
 
+static gboolean
+check_high_alarm_limit(GkrellmAlert *alert, gfloat value)
+	{
+	GkrellmTrigger	*trigger = &alert->high;
+
+	if (value < trigger->alarm_limit)
+		{
+		trigger->alarm_delay = alert->delay;
+		return FALSE;
+		}
+	if (trigger->alarm_delay == 0)
+		{
+		if (trigger->warn_delay > 0)
+			trigger->warn_delay -= 1;
+		return TRUE;
+		}
+	trigger->alarm_delay -= 1;
+	return FALSE;
+	}
+
+static gboolean
+check_high_warn_limit(GkrellmAlert *alert, gfloat value)
+	{
+	GkrellmTrigger	*trigger = &alert->high;
+
+	if (value < trigger->warn_limit)
+		{
+		trigger->warn_delay = alert->delay;
+		return FALSE;
+		}
+	if (trigger->warn_delay == 0)
+		return TRUE;
+	trigger->warn_delay -= 1;
+	return FALSE;
+	}
+
+static gboolean
+check_low_warn_limit(GkrellmAlert *alert, gfloat value)
+	{
+	GkrellmTrigger	*trigger = &alert->low;
+
+	if (value > trigger->warn_limit)
+		{
+		trigger->warn_delay = alert->delay;
+		return FALSE;
+		}
+	if (trigger->warn_delay == 0)
+		return TRUE;
+	trigger->warn_delay -= 1;
+	return FALSE;
+	}
+
+static gboolean
+check_low_alarm_limit(GkrellmAlert *alert, gfloat value)
+	{
+	GkrellmTrigger	*trigger = &alert->low;
+
+	if (value > trigger->alarm_limit)
+		{
+		trigger->alarm_delay = alert->delay;
+		return FALSE;
+		}
+	if (trigger->alarm_delay == 0)
+		{
+		if (trigger->warn_delay > 0)
+			trigger->warn_delay -= 1;
+		return TRUE;
+		}
+	trigger->alarm_delay -= 1;
+	return FALSE;
+	}
+
 void
 gkrellm_check_alert(GkrellmAlert *alert, gfloat value)
 	{
 	if (!alert || !alert->activated || alert->freeze || !_GK.initialized)
 		return;
+	if (alert->delay > 0 && !GK.second_tick)
+		return;
+
 	if (alert->check_low)
 		{
-		if (value <= alert->low.alarm_limit)
+		if (check_low_alarm_limit(alert, value))
 			{				
 			stop_alarm(alert, &alert->high);
 			stop_warn(alert, &alert->high);
@@ -233,7 +332,7 @@ gkrellm_check_alert(GkrellmAlert *alert, gfloat value)
 		else
 			{
 			stop_alarm(alert, &alert->low);
-			if (value <= alert->low.warn_limit)
+			if (check_low_warn_limit(alert, value))
 				{
 				stop_alarm(alert, &alert->high);
 				stop_warn(alert, &alert->high);
@@ -245,7 +344,7 @@ gkrellm_check_alert(GkrellmAlert *alert, gfloat value)
 		}
 	if (alert->check_high)
 		{
-		if (value >= alert->high.alarm_limit)
+		if (check_high_alarm_limit(alert, value))
 			{
 			stop_warn(alert, &alert->high);
 			stop_warn(alert, &alert->low);
@@ -255,7 +354,7 @@ gkrellm_check_alert(GkrellmAlert *alert, gfloat value)
 		else
 			{
 			stop_alarm(alert, &alert->high);
-			if (value >= alert->high.warn_limit)
+			if (check_high_warn_limit(alert, value))
 				{
 				stop_warn(alert, &alert->low);
 				stop_alarm(alert, &alert->low);
@@ -289,6 +388,8 @@ gkrellm_alert_stop_connect(GkrellmAlert *alert, void (*func)(), gpointer data)
 static void
 destroy_alert(GkrellmAlert *alert)
 	{
+	if (!alert)
+		return;
 	if (g_list_find(gkrellm_alert_list, alert))
 		{
 		gkrellm_alert_list = g_list_remove(gkrellm_alert_list, alert);
@@ -298,6 +399,7 @@ destroy_alert(GkrellmAlert *alert)
 	g_free(alert->unit_string);
 	g_free(alert->alarm_command);
 	g_free(alert->warn_command);
+	g_free(alert->voice_string);
 	g_free(alert);
 	}
 
@@ -322,7 +424,7 @@ gkrellm_alert_create(GkrellmPanel *p, gchar *name, gchar *unit_string,
 	alert = g_new0(GkrellmAlert, 1);
 	alert->panel = p;
 	alert->name = g_strdup(name);
-	alert->unit_string = g_strdup(unit_string ? unit_string : "");
+	alert->unit_string = g_strdup(unit_string);
 	alert->check_high = check_high;
 	alert->check_low = check_low;
 	alert->do_panel_updates = do_updates;
@@ -331,6 +433,10 @@ gkrellm_alert_create(GkrellmPanel *p, gchar *name, gchar *unit_string,
 	alert->step0 = step0;
 	alert->step1 = step1;
 	alert->digits = digits;
+
+	gkrellm_alert_set_triggers(alert, min_low, min_low, min_low, min_low);
+	alert->activated = FALSE;
+
 	return alert;
 	}
 
@@ -354,6 +460,57 @@ gkrellm_alert_set_triggers(GkrellmAlert *alert,
 	alert->activated = TRUE;
 	}
 
+void
+gkrellm_alert_set_delay(GkrellmAlert *alert, gint delay)
+	{
+	if (!alert)
+		return;
+	alert->delay = delay;
+	alert->low.warn_delay = alert->low.alarm_delay = delay;
+	alert->high.warn_delay = alert->high.alarm_delay = delay;
+	}
+
+void
+gkrellm_alert_delay_config(GkrellmAlert *alert, gint step,
+			gint high, gint low)
+	{
+	if (!alert || step < 1 || high < step)
+		return;
+	low = (low / step) * step;
+	high = (high / step) * step;
+
+	alert->delay_high = (gfloat) high;
+	alert->delay_low = (gfloat) low;
+	alert->delay_step = (gfloat) step;
+	gkrellm_alert_set_delay(alert, low);
+	}
+
+void
+gkrellm_alert_dup(GkrellmAlert **a_dst, GkrellmAlert *a_src)
+	{
+	GkrellmAlert	*alert;
+
+	if (!a_dst || !a_src)
+		return;
+	gkrellm_alert_destroy(a_dst);
+	alert = gkrellm_alert_create(a_src->panel, a_src->name,
+				a_src->unit_string, a_src->check_high, a_src->check_low,
+				a_src->do_panel_updates, a_src->max_high, a_src->min_low,
+				a_src->step0, a_src->step1, a_src->digits);
+	gkrellm_dup_string(&alert->alarm_command, a_src->alarm_command);
+	gkrellm_dup_string(&alert->warn_command, a_src->warn_command);
+	gkrellm_dup_string(&alert->voice_string, a_src->voice_string);
+	alert->warn_repeat_set = a_src->warn_repeat_set;
+	alert->alarm_repeat_set = a_src->alarm_repeat_set;
+	alert->voice_repeat_set = a_src->voice_repeat_set;
+	alert->low = a_src->low;
+	alert->high = a_src->high;
+	gkrellm_alert_set_delay(alert, a_src->delay);
+
+	alert->activated = a_src->activated;
+	*a_dst = alert;
+	}
+
 static void
 reset_alert(GkrellmAlert *alert)
 	{
@@ -461,24 +618,42 @@ update_alerts(void)
 			{
 			if (alert->alarm_repeat > 0 && --alert->alarm_repeat == 0)
 				{
-				run_command(alert->alarm_command);
+				run_command(alert, alert->alarm_command);
 				alert->alarm_repeat = alert->alarm_repeat_set;
 				}
 			if (alert->warn_repeat > 0 && --alert->warn_repeat == 0)
 				{
-				run_command(alert->warn_command);
+				run_command(alert, alert->warn_command);
 				alert->warn_repeat = alert->warn_repeat_set;
 				}
 			if (alert->voice_repeat > 0 && --alert->voice_repeat == 0)
 				{
-				send_gvoice(alert->voice_string);
+				send_gvoice(alert, alert->voice_string);
 				alert->voice_repeat = alert->voice_repeat_set;
 				}
 			}
 		}
 	}
 
+gboolean
+gkrellm_alert_is_activated(GkrellmAlert *alert)
+	{
+	if (alert)
+		return alert->activated;
+	return FALSE;
+	}
+
 /* ------------------------------------------------------------ */
+void
+gkrellm_alert_command_process_connect(GkrellmAlert *alert, void (*func)(),
+						gpointer data)
+	{
+	if (!alert)
+		return;
+	alert->cb_command_process = func;
+	alert->cb_command_process_data = data;
+	}
+
 void
 gkrellm_alert_config_connect(GkrellmAlert *alert, void (*func)(),gpointer data)
 	{
@@ -488,6 +663,16 @@ gkrellm_alert_config_connect(GkrellmAlert *alert, void (*func)(),gpointer data)
 	alert->cb_config_data = data;
 	}
 
+void
+gkrellm_alert_config_create_connect(GkrellmAlert *alert,
+						void (*func)(), gpointer data)
+	{
+	if (!alert)
+		return;
+	alert->cb_config_create = func;
+	alert->cb_config_create_data = data;
+	}
+
 static void
 alert_delete(GtkWidget *widget, GkrellmAlert **ap)
 	{
@@ -519,6 +704,7 @@ alert_close(GtkWidget *widget, GkrellmAlert **alert)
 		{
 		gtk_widget_destroy(ap->config_window);
 		ap->config_window = NULL;
+		ap->delete_button = NULL;
 		}
 	}
 
@@ -542,6 +728,7 @@ alert_apply(GtkWidget *widget, GkrellmAlert **ap)
 	GkrellmAlert	*alert;
 	gchar			*s;
 	GtkSpinButton	*spin;
+	gint			n;
 
 	alert = *ap;
 	if (alert->check_high)
@@ -589,7 +776,16 @@ alert_apply(GtkWidget *widget, GkrellmAlert **ap)
 			alert->voice_repeat_set = 0;
 		alert->voice_repeat = alert->voice_repeat_set;
 		}
+
+	if (alert->delay_spin_button)
+		{
+		spin = GTK_SPIN_BUTTON(alert->delay_spin_button);
+		n = gtk_spin_button_get_value_as_int(spin);
+		gkrellm_alert_set_delay(alert, n / alert->delay_step);
+		}
+
 	alert->activated = TRUE;
+	gtk_widget_set_sensitive(alert->delete_button, alert->activated);
 	if (alert->cb_config)
 		(*alert->cb_config)(alert, alert->cb_config_data);
 	gkrellm_config_modified();
@@ -602,6 +798,21 @@ alert_ok(GtkWidget *widget, GkrellmAlert **alert)
 	alert_close(NULL, alert);
 	}
 
+static void
+cb_delay_spin_changed(GtkAdjustment *adjustment, GkrellmAlert *alert)
+	{
+	GtkSpinButton	*spin;
+	gint			delay;
+
+	spin = GTK_SPIN_BUTTON(alert->delay_spin_button);
+	delay = gtk_spin_button_get_value_as_int(spin);
+	if ((delay % alert->delay_step) != 0)
+		{
+		delay = delay / alert->delay_step * alert->delay_step;
+		gtk_spin_button_set_value(spin, delay);
+		}
+	}
+
 static void
 cb_high_alarm_spin_changed(GtkAdjustment *adjustment, GkrellmAlert *alert)
 	{
@@ -682,6 +893,7 @@ gkrellm_alert_config_window(GkrellmAlert **alert)
 	GtkWidget	*separator;
 	GkrellmAlert *ap;
 	gchar		*title;
+	gint		w;
 
 	if (!alert || !*alert)
 		return;
@@ -697,13 +909,22 @@ gkrellm_alert_config_window(GkrellmAlert **alert)
 		gtk_container_set_border_width(GTK_CONTAINER(ap->config_window), 4);
 		main_vbox = gtk_vbox_new(FALSE, 0);
 		gtk_container_add(GTK_CONTAINER(ap->config_window), main_vbox);
-		
-		title = g_strdup_printf("%s - %s", ap->name, ap->unit_string);
+
+		if (ap->name && ap->unit_string)
+			title = g_strdup_printf("%s - %s", ap->name, ap->unit_string);
+		else if (ap->name)
+			title = g_strdup_printf("%s", ap->name);
+		else
+			title = g_strdup_printf("%s", ap->unit_string);
+
 		vbox = gkrellm_gtk_framed_vbox(main_vbox, title, 4, FALSE, 4, 3);
 		g_free(title);
 
 		hbox = gtk_hbox_new(FALSE, 0);
 		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+		w = 70;
+		if (ap->max_high > 100000)
+			w += log(ap->max_high / 100000) * 5;
 		if (ap->check_high)
 			{
 			vbox1 = gkrellm_gtk_framed_vbox(hbox, _("High Limits"),
@@ -711,13 +932,13 @@ gkrellm_alert_config_window(GkrellmAlert **alert)
 			gkrellm_gtk_spin_button(vbox1, &ap->high.alarm_limit_spin_button,
 					ap->high.alarm_limit, ap->min_low, ap->max_high,
 					ap->step0, ap->step1,
-					ap->digits, 70, cb_high_alarm_spin_changed, ap, FALSE,
+					ap->digits, w, cb_high_alarm_spin_changed, ap, FALSE,
 					_("High alarm limit"));
 		
 			gkrellm_gtk_spin_button(vbox1, &ap->high.warn_limit_spin_button,
 					ap->high.warn_limit, ap->min_low, ap->max_high,
 					ap->step0, ap->step1,
-					ap->digits, 70, cb_high_warn_spin_changed, ap, FALSE,
+					ap->digits, w, cb_high_warn_spin_changed, ap, FALSE,
 					_("High warn limit"));
 			}
 
@@ -728,14 +949,29 @@ gkrellm_alert_config_window(GkrellmAlert **alert)
 			gkrellm_gtk_spin_button(vbox1, &ap->low.warn_limit_spin_button,
 					ap->low.warn_limit, ap->min_low, ap->max_high,
 					ap->step0, ap->step1,
-					ap->digits, 70, cb_low_warn_spin_changed, ap, FALSE,
+					ap->digits, w, cb_low_warn_spin_changed, ap, FALSE,
 					_("Low warn limit"));
 			gkrellm_gtk_spin_button(vbox1, &ap->low.alarm_limit_spin_button,
 					ap->low.alarm_limit, ap->min_low, ap->max_high,
 					ap->step0, ap->step1,
-					ap->digits, 70, cb_low_alarm_spin_changed, ap, FALSE,
+					ap->digits, w, cb_low_alarm_spin_changed, ap, FALSE,
 					_("Low alarm limit"));
 			}
+		if (ap->delay_step > 0)
+			{
+			vbox1 = gkrellm_gtk_framed_vbox(vbox, NULL, 2, FALSE, 2, 2);
+			gkrellm_gtk_spin_button(vbox1, &ap->delay_spin_button,
+					ap->delay * ap->delay_step, ap->delay_low, ap->delay_high,
+					ap->delay_step, ap->delay_step,
+					0, 70, cb_delay_spin_changed, ap, FALSE,
+					_("Seconds limit conditions must exist to have an alert"));
+			}
+		if (ap->cb_config_create)
+			{
+			vbox1 = gkrellm_gtk_framed_vbox(vbox, NULL, 2, FALSE, 2, 2);
+			(*ap->cb_config_create)(alert, vbox1, ap->cb_config_create_data);
+			}
+
 		vbox1 = gkrellm_gtk_framed_vbox(vbox,
 				_("Commands - with repeat intervals in seconds"),
 				2, FALSE, 2, 2);
@@ -819,29 +1055,30 @@ gkrellm_alert_config_window(GkrellmAlert **alert)
 		gtk_box_set_spacing(GTK_BOX(hbox), 5);
 		gtk_box_pack_start(GTK_BOX(main_vbox), hbox, FALSE, FALSE, 0);
 
-		button = gtk_button_new_from_stock(GTK_STOCK_DELETE);
-		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-		g_signal_connect(G_OBJECT(button), "clicked",
+		ap->delete_button = gtk_button_new_from_stock(GTK_STOCK_DELETE);
+		GTK_WIDGET_SET_FLAGS(ap->delete_button, GTK_CAN_DEFAULT);
+		g_signal_connect(G_OBJECT(ap->delete_button), "clicked",
 				G_CALLBACK(alert_delete), alert);
-	    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+		gtk_box_pack_start(GTK_BOX(hbox), ap->delete_button, TRUE, TRUE, 0);
+		gtk_widget_set_sensitive(ap->delete_button, ap->activated);
 
 		button = gtk_button_new_from_stock(GTK_STOCK_APPLY);
 		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 		g_signal_connect(G_OBJECT(button), "clicked",
 				G_CALLBACK(alert_apply), alert);
-	    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+		gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
 
 		button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
 		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 		g_signal_connect(G_OBJECT(button), "clicked",
 				G_CALLBACK(alert_close), alert);
-	    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+		gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
 
 		button = gtk_button_new_from_stock(GTK_STOCK_OK);
 		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 		g_signal_connect(G_OBJECT(button), "clicked",
 				G_CALLBACK(alert_ok), alert);
-	    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
+		gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
 		gtk_widget_grab_default(button);
 
 		gtk_widget_show_all(ap->config_window);
@@ -877,6 +1114,10 @@ gkrellm_save_alertconfig(FILE *f, GkrellmAlert *alert,
 	fprintf(f, "%s limits %.2f %.2f %.2f %.2f\n", s,
 			alert->high.alarm_limit, alert->high.warn_limit,
 			alert->low.warn_limit, alert->low.alarm_limit);
+	if (alert->delay_step > 0)
+		fprintf(f, "%s delay %d %d %d %d\n", s, alert->delay,
+					alert->delay_high, alert->delay_low, alert->delay_step);
+
 	g_free(s);
 	}
 
@@ -906,6 +1147,12 @@ gkrellm_load_alertconfig(GkrellmAlert **ap, gchar *config_line)
 	else if (!strcmp(config, "values"))
 		sscanf(item, "%d %d %d %d", &alert->do_panel_updates,
 				&alert->check_high, &alert->check_low, &alert->check_boolean);
+	else if (!strcmp(config, "delay"))
+		{
+		sscanf(item, "%d %d %d %d", &n,
+				&alert->delay_high, &alert->delay_low, &alert->delay_step);
+		gkrellm_alert_set_delay(alert, n);
+		}
 	else if (!strcmp(config, "repeat"))
 		sscanf(item, "%d %d %d", &alert->alarm_repeat_set,
 				&alert->warn_repeat_set, &alert->voice_repeat_set);
