@@ -721,9 +721,11 @@ save_battery_config(FILE *f)
 	fprintf(f, "%s full_cap_fallback %d\n", BAT_CONFIG_KEYWORD,
 						full_cap_fallback);
 	fprintf(f, "%s poll_interval %d\n", BAT_CONFIG_KEYWORD, poll_interval);
-	fprintf(f, "%s launch1 %s\n", BAT_CONFIG_KEYWORD, launch.command);
-	fprintf(f, "%s tooltip_comment %s\n",
-				BAT_CONFIG_KEYWORD, launch.tooltip_comment);
+	if (launch.command)
+		fprintf(f, "%s launch1 %s\n", BAT_CONFIG_KEYWORD, launch.command);
+	if (launch.tooltip_comment)
+		fprintf(f, "%s tooltip_comment %s\n",
+					BAT_CONFIG_KEYWORD, launch.tooltip_comment);
 	fprintf(f, "%s alert_units_percent %d\n", BAT_CONFIG_KEYWORD,
 				alert_units_are_percent);
 	for (n = 0, list = battery_list; list; list = list->next, ++n)
@@ -816,7 +818,7 @@ update_battery_panels(void)
 	}
 
 static void
-cb_battery_set_alert(GtkWidget *entry, Battery *bat)
+cb_set_alert(GtkWidget *button, Battery *bat)
 	{
 	create_battery_alert(bat, FALSE);
 	gkrellm_alert_config_window(&bat->alert);
@@ -1014,14 +1016,13 @@ create_battery_tab(GtkWidget *tab_vbox)
 		gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 3);
 		button = gtk_button_new();
 		g_signal_connect(G_OBJECT(button), "clicked",
-					G_CALLBACK(cb_battery_set_alert),
-					(Battery *) list->data);
+					G_CALLBACK(cb_set_alert), (Battery *) list->data);
 		gtk_widget_show_all(hbox2);
 		gtk_container_add(GTK_CONTAINER(button), hbox2);
 		gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 4);
 
 //		gkrellm_gtk_button_connected(hbox, NULL, FALSE, FALSE, 0,
-//				cb_battery_set_alert, NULL, _("Alerts"));
+//				cb_set_alert, NULL, _("Alerts"));
 		}
 
 	vbox1 = gkrellm_gtk_framed_vbox_end(vbox, _("Launch Commands"),
