@@ -71,7 +71,9 @@ typedef struct
 	}
 	KeyTable;
 
-static gint		input_id;
+static gint		client_input_id,
+				client_fd;
+
 static gchar	server_buf[4097];
 static gint		buf_index;
 
@@ -1161,6 +1163,22 @@ client_server_decimal_point(gchar *line)
 	locale_sync();
 	}
 
+static void
+client_server_io_timeout(gchar *line)
+	{
+	sscanf(line, "%d", &_GK.client_server_io_timeout);
+	if (_GK.client_server_io_timeout < 2)
+		_GK.client_server_io_timeout = 0;
+	}
+
+static void
+client_server_reconnect_timeout(gchar *line)
+	{
+	sscanf(line, "%d", &_GK.client_server_reconnect_timeout);
+	if (_GK.client_server_reconnect_timeout < 2)
+		_GK.client_server_reconnect_timeout = 0;
+	}
+
 /* ================================================================= */
 KeyTable	setup_table[] =
 	{
@@ -1178,6 +1196,8 @@ KeyTable	setup_table[] =
 	{"<monitors>",		client_monitor_setup },
 	{"<decimal_point>",	client_server_decimal_point },
 	{"<error>",			client_server_error },
+	{"<io_timeout>",	client_server_io_timeout},
+	{"<reconnect_timeout>", client_server_reconnect_timeout},
 	};
 
 KeyTable	update_table[] =
@@ -1196,9 +1216,9 @@ KeyTable	update_table[] =
 	{"<apm>",			client_battery_line_from_server},
 	{"<battery>",		client_battery_line_from_server},
 	{"<sensors>",		client_sensors_line_from_server},
-//	{"<>",			client__line_from_server},
 	{"<time>",			client_time_line_from_server},
 	{"<uptime>",		client_uptime_line_from_server},
+//	{"<>",			client__line_from_server},
 	};
 
 
@@ -1283,6 +1303,7 @@ read_server_setup(gint fd)
 	|  locale_sync() here.
 	*/
 	locale_sync();
+	_GK.client_server_read_time = time(0);
 	table_size = sizeof(setup_table) / sizeof(KeyTable);
 	while (1)
 		{
@@ -1310,6 +1331,16 @@ read_server_setup(gint fd)
 	setup_done = TRUE;
 	}
 
+void
+gkrellm_client_mode_disconnect(void)
+	{
+	if (client_fd >= 0)
+		close(client_fd);
+	client_fd = -1;
+	gdk_input_remove(client_input_id);
+	client_input_id = 0;
+	}
+
 static void
 read_server_input(gpointer data, gint fd, GdkInputCondition condition)
 	{
@@ -1320,12 +1351,11 @@ read_server_input(gpointer data, gint fd, GdkInputCondition condition)
 	count = recv(fd, server_buf + buf_index, n, 0);
 	if (count <= 0)
 		{
-		close(fd);
-		gdk_input_remove(input_id);
-		input_id = 0;
-		/* reconnect */
+		gkrellm_client_mode_disconnect();
 		return;
 		}
+	if (_GK.time_now > 0)
+		_GK.client_server_read_time = _GK.time_now;
 	server_buf[buf_index + count] = '\0';
 	line = server_buf;
 	table_size = sizeof(update_table) / sizeof(KeyTable);
@@ -1349,9 +1379,13 @@ read_server_input(gpointer data, gint fd, GdkInputCondition condition)
 	}
 
 static gint
-server_shutdown(gint fd, gchar *message)
+unable_to_connect(void)
 	{
-	printf("%s\n", message);
+	static gboolean	warned;
+
+	if (!warned)
+		printf("%s\n", _("Unable to connect."));
+	warned = TRUE;
 	return -1;
 	}
 
@@ -1375,7 +1409,7 @@ connect_to_server(void)
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_protocol = 0;
 	if ((rv = getaddrinfo(_GK.server, portnumber, &hints, &res0)) != 0)
-		return server_shutdown(fd, _("Unable to connect."));
+		return unable_to_connect();
 
 	for (res = res0; res; res = res->ai_next)
 		{
@@ -1393,7 +1427,7 @@ connect_to_server(void)
 		}
 	freeaddrinfo(res0);
 	if (fd == -1)
-		return server_shutdown(fd, _("Unable to connect."));
+		return unable_to_connect();
 #else
 	if (_GK.debug_level & DEBUG_CLIENT)
 		printf("\t[connect_to_server: %s:%d]\n", _GK.server, _GK.server_port);
@@ -1414,7 +1448,7 @@ connect_to_server(void)
 			}
 		}
 	if (fd < 0) 
-		return server_shutdown(fd, _("Unable to connect."));
+		return unable_to_connect();
 #endif
 
 	return fd;
@@ -1424,22 +1458,21 @@ gboolean
 gkrellm_client_mode_connect(void)
 	{
 	gchar	buf[128];
-	gint	fd;
 
 	if (_GK.server_port == 0)
 		_GK.server_port = GKRELLMD_SERVER_PORT;
 
-	if ((fd = connect_to_server()) < 0)
+	if ((client_fd = connect_to_server()) < 0)
 		return FALSE;
 
 	snprintf(buf, sizeof(buf), "gkrellm %d.%d.%d %s\n",
 			GKRELLM_VERSION_MAJOR, GKRELLM_VERSION_MINOR,
 			GKRELLM_VERSION_REV, GKRELLM_EXTRAVERSION);
-	send(fd, buf, strlen(buf), 0);
+	send(client_fd, buf, strlen(buf), 0);
 
 	/* Initial setup lines from server are read in blocking mode.
 	*/
-	read_server_setup(fd);
+	read_server_setup(client_fd);
 
 	/* Extra stuff not handled in read_server_setup()
 	*/
@@ -1447,11 +1480,10 @@ gkrellm_client_mode_connect(void)
 
 	/* Now switch to non blocking and set up a read handler.
 	*/
-	fcntl(fd, F_SETFL, O_NONBLOCK);
-	input_id = gdk_input_add(fd, GDK_INPUT_READ,
+	fcntl(client_fd, F_SETFL, O_NONBLOCK);
+	client_input_id = gdk_input_add(client_fd, GDK_INPUT_READ,
 					(GdkInputFunction) read_server_input, NULL);
 
-
 	return TRUE;
 	}
 
@@ -1461,11 +1493,14 @@ static gboolean	client_mode_thread_busy;
 gint
 gkrellm_client_server_connect_state(void)
 	{
-	if (client_mode_thread_busy)
+	if (client_mode_thread_busy)			/* reconnect in progress? */
 		return 2;
-	if (_GK.client_mode && input_id > 0)
+	if (_GK.client_mode && client_input_id > 0)	/* Currently connected? */
 		return 1;
-	return 0;
+	else if (_GK.client_mode)
+		return 0;
+	else
+		return -1;
 	}
 
 static gpointer
