@@ -58,6 +58,10 @@ typedef struct
 					show_fan;
 	gint			save_label_position;  /* When toggling sensors on/off */
 
+	GkrellmAlert	*alert;
+	gulong			previous_alert_value,
+					previous_alert_total;
+
 	GkrellmLauncher	launch;
 	GtkWidget		*launch_entry,
 					*tooltip_entry;
@@ -79,6 +83,9 @@ GList			*cpu_mon_list,
 void			(*read_cpu_data)();
 static CpuMon	*composite_cpu;
 
+static GkrellmAlert	*cpu_alert;		/* One alert dupped for each CPU */
+static gboolean	alert_includes_nice;
+
 static gint		smp_mode;
 static gboolean	cpu_enabled  = TRUE;
 static gboolean	omit_nice_mode,
@@ -252,14 +259,18 @@ gkrellm_cpu_stats(gint n, gulong *user, gulong *nice,
 /* ======================================================================== */
 
 static void
-format_chart_text(CpuMon *cpu, gchar *buf, gint size)
+format_cpu_data(CpuMon *cpu, gchar *src_string, gchar *buf, gint size)
 	{
 	GkrellmChart	*cp;
 	gchar			c, *s;
 	gint			len, sys, user, nice = 0, total, t;
 
+	if (!buf || size < 1)
+		return;
 	--size;
 	*buf = '\0';
+	if (!src_string)
+		return;
 	cp = cpu->chart;
 	sys = gkrellm_get_current_chartdata(cpu->sys_cd);
 	user = gkrellm_get_current_chartdata(cpu->user_cd);
@@ -270,7 +281,7 @@ format_chart_text(CpuMon *cpu, gchar *buf, gint size)
 		if (!omit_nice_mode && !gkrellm_get_chartdata_hide(cpu->nice_cd))
 			total += nice;
 		}
-	for (s = text_format_locale; *s != '\0' && size > 0; ++s)
+	for (s = src_string; *s != '\0' && size > 0; ++s)
 		{
 		len = 1;
 		if (*s == '$' && *(s + 1) != '\0')
@@ -286,6 +297,8 @@ format_chart_text(CpuMon *cpu, gchar *buf, gint size)
 				t = nice;
 			else if (c == 'L')
 				len = snprintf(buf, size, "%s", cp->panel->label->string);
+			else if (c == 'H')
+				len = snprintf(buf, size, "%s", gkrellm_sys_get_host_name());
 			else
 				{
 				*buf = *s;
@@ -323,10 +336,17 @@ draw_cpu_extra(CpuMon *cpu)
 
 	if (!cp)
 		return;
-	format_chart_text(cpu, buf, sizeof(buf));
+	format_cpu_data(cpu, text_format_locale, buf, sizeof(buf));
 	gkrellm_draw_chart_text(cp, style_id, buf);
 	}
 
+static void
+cb_command_process(GkrellmAlert *alert, gchar *src, gchar *dst, gint len,
+				CpuMon *cpu)
+    {
+    format_cpu_data(cpu, src, dst, len);
+	}
+
 static void
 refresh_cpu_chart(CpuMon *cpu)
 	{
@@ -355,7 +375,7 @@ draw_sensor_decals(CpuMon *cpu)
 		}
 	else
 		{
-		toggle = time_now & 2;
+		toggle = _GK.time_now & 2;
 		if (cpu->show_fan && (toggle || !cpu->show_temperature) )
 			{
 			gkrellm_sensor_read_fan(cpu->sensor_fan, &f);
@@ -395,6 +415,7 @@ update_cpu(void)
 	GkrellmPanel	*p;
 	GkrellmKrell	*krell;
 	gulong			total, krell_value, full_scale;
+	gulong			alert_value, alert_total_diff;
 
 	_GK.cpu_sys_activity = 0;
 	if (composite_cpu)
@@ -422,6 +443,17 @@ update_cpu(void)
 			{
 			gkrellm_store_chartdata(cp, total, cpu->sys, cpu->user, cpu->nice);
 			refresh_cpu_chart(cpu);
+
+			alert_value = cpu->sys + cpu->user;
+			if (alert_includes_nice)
+				alert_value += cpu->nice;
+			alert_total_diff = total - cpu->previous_alert_total;
+			if (alert_total_diff > 0)
+				gkrellm_check_alert(cpu->alert,
+					100.0 * (gfloat) (alert_value - cpu->previous_alert_value)
+					/ (gfloat) alert_total_diff);
+			cpu->previous_alert_value = alert_value;
+			cpu->previous_alert_total = total;
 			}
 		if (   (GK.two_second_tick && !sensor_separate_mode)
 			|| (GK.five_second_tick && sensor_separate_mode)
@@ -433,12 +465,15 @@ update_cpu(void)
 		if (full_scale > 0)		/* Can be 0 for data from gkrellmd */
 			gkrellm_set_krell_full_scale(krell, (gint) full_scale, 10);
 
+		krell_value = cpu->sys + cpu->user;
+
 		if (cpu->previous_total > 0)
 			{
-			krell_value = cpu->sys + cpu->user;
 			if (!omit_nice_mode && !gkrellm_get_chartdata_hide(cpu->nice_cd))
 				krell_value += cpu->nice;
 			gkrellm_update_krell(p, krell, krell_value);
+			gkrellm_panel_label_on_top_of_decals(p,
+						gkrellm_alert_decal_visible(cpu->alert));
 			gkrellm_draw_panel_layers(p);
 			}
 		cpu->previous_total = total;
@@ -856,8 +891,73 @@ create_cpu(GtkWidget *vbox, gint first_create)
 
 static GtkWidget	*text_format_combo;
 static GtkWidget	*smp_button[3];
+static GtkWidget	*alert_config_nice_button;
 
 
+static void
+cb_alert_trigger(GkrellmAlert *alert, CpuMon *cpu)
+	{
+	GkrellmAlertdecal	*ad;
+	GkrellmDecal		*ds, *df;
+
+	alert->panel = cpu->chart->panel;
+	ds = cpu->sensor_decal;
+	df = cpu->fan_decal;
+	if (gkrellm_is_decal_visible(ds) && !gkrellm_is_decal_visible(df))
+		{
+		ad = &alert->ad;
+		ad->x = 0;
+		ad->y = ds->y - 1;
+		ad->w = ds->x - 1;
+		ad->h = ds->h + 2;
+		gkrellm_render_default_alert_decal(alert);
+		}
+	}
+
+static void
+dup_cpu_alert(void)
+	{
+	GList	*list;
+	CpuMon	*cpu;
+
+	for (list = cpu_mon_list->next; list; list = list->next)
+		{
+		cpu = (CpuMon *) list->data;
+		gkrellm_alert_dup(&cpu->alert, cpu_alert);
+		gkrellm_alert_trigger_connect(cpu->alert, cb_alert_trigger, cpu);
+		gkrellm_alert_command_process_connect(cpu->alert,
+					cb_command_process, cpu);
+		}
+	}
+
+static void
+create_alert(void)
+	{
+	GList		*list;
+	CpuMon		*cpu;
+	static void	cb_alert_config(GkrellmAlert *ap, gpointer data);
+#if !defined(WIN32)
+	static void	cb_alert_config_create(GkrellmAlert *ap, GtkWidget *vbox,
+						gpointer data);
+#endif
+
+	list = g_list_nth(cpu_mon_list, 0);
+	if (!list)
+		return;
+	cpu = (CpuMon *) list->data;
+	cpu_alert = gkrellm_alert_create(NULL, _("CPU"),
+					_("Percent Usage"),
+					TRUE, FALSE, TRUE,
+					100, 10, 1, 10, 0);
+	gkrellm_alert_delay_config(cpu_alert, 1, 60 * 60, 2);
+	gkrellm_alert_config_connect(cpu_alert, cb_alert_config, NULL);
+#if !defined(WIN32)
+	gkrellm_alert_config_create_connect(cpu_alert,
+							cb_alert_config_create, NULL);
+#endif
+	/* This alert is a master to be dupped and is itself never checked */
+	}
+
 static void
 save_cpu_config(FILE *f)
 	{
@@ -885,6 +985,12 @@ save_cpu_config(FILE *f)
 	fprintf(f, "%s sensor_mode %d\n", CPU_CONFIG_KEYWORD,
 			sensor_separate_mode);
 	fprintf(f, "%s text_format %s\n", CPU_CONFIG_KEYWORD, text_format);
+	if (cpu_alert)
+		{
+		gkrellm_save_alertconfig(f, cpu_alert, CPU_CONFIG_KEYWORD, NULL);
+		fprintf(f, "%s alert_includes_nice %d\n", CPU_CONFIG_KEYWORD,
+					alert_includes_nice);
+		}
 	}
 
 static void
@@ -911,6 +1017,8 @@ load_cpu_config(gchar *arg)
 			sscanf(item, "%d\n", &sensor_separate_mode);
 		else if (!strcmp(config, "text_format"))
 			gkrellm_locale_dup_string(&text_format, item, &text_format_locale);
+		else if (!strcmp(config, "alert_includes_nice"))
+			sscanf(item, "%d\n", &alert_includes_nice);
 		else if (!strcmp(config, GKRELLM_CHARTCONFIG_KEYWORD))
 			{
 			sscanf(item, "%31s %[^\n]", cpu_name, command);
@@ -922,6 +1030,13 @@ load_cpu_config(gchar *arg)
 							nice_time_unsupported ? 2 : 3);
 				}
 			}
+		else if (!strcmp(config, GKRELLM_ALERTCONFIG_KEYWORD))
+			{
+			if (!cpu_alert)
+				create_alert();
+			gkrellm_load_alertconfig(&cpu_alert, item);
+			dup_cpu_alert();
+			}
 		else if (!strcmp(config, "extra_info"))
 			{
 			sscanf(item, "%31s %[^\n]", cpu_name, command);
@@ -955,6 +1070,33 @@ load_cpu_config(gchar *arg)
 		}
 	}
 
+static void
+cb_alert_config(GkrellmAlert *ap, gpointer data)
+	{
+#if !defined(WIN32)
+	alert_includes_nice = GTK_TOGGLE_BUTTON(alert_config_nice_button)->active;
+#endif
+	dup_cpu_alert();
+	}
+
+#if !defined(WIN32)
+static void
+cb_alert_config_create(GkrellmAlert *ap, GtkWidget *vbox, gpointer data)
+	{
+	gkrellm_gtk_check_button(vbox, &alert_config_nice_button,
+			alert_includes_nice, FALSE, 2,
+			_("nice time"));
+	}
+#endif
+
+static void
+cb_set_alert(GtkWidget *button, gpointer data)
+	{
+	if (!cpu_alert)
+		create_alert();
+	gkrellm_alert_config_window(&cpu_alert);
+	}
+
 static gboolean
 fix_panel(CpuMon *cpu)
 	{
@@ -1083,6 +1225,8 @@ N_("\t$T    total CPU time percent usage\n"),
 N_("\t$s    sys time percent usage\n"),
 N_("\t$u    user time percent usage\n"),
 N_("\t$n    nice time percent usage\n"),
+"\n",
+N_("Substitution variables may be used in alert commands.\n")
 };
 
 static void
@@ -1090,7 +1234,7 @@ create_cpu_tab(GtkWidget *tab_vbox)
 	{
 	GtkWidget	*tabs;
 	GtkWidget	*button;
-	GtkWidget	*hbox, *vbox, *vbox1;
+	GtkWidget	*hbox, *hbox1, *vbox, *vbox1;
 	GtkWidget	*text;
 	GtkWidget	*table;
 	GSList		*group;
@@ -1156,13 +1300,15 @@ create_cpu_tab(GtkWidget *tab_vbox)
 
 /* -- Setup tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Setup"));
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Format String for Chart Labels"),
-			 4, FALSE, 0, 2);
 	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 5);
+	vbox1 = gkrellm_gtk_framed_vbox(hbox, _("Format String for Chart Labels"),
+			 4, FALSE, 0, 2);
+	hbox1 = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, TRUE, 5);
 	text_format_combo = gtk_combo_new();
-	gtk_widget_set_size_request (GTK_WIDGET(text_format_combo), 350, -1);
-	gtk_box_pack_start(GTK_BOX(hbox), text_format_combo, FALSE, TRUE, 0);
+	gtk_widget_set_size_request (GTK_WIDGET(text_format_combo), 300, -1);
+	gtk_box_pack_start(GTK_BOX(hbox1), text_format_combo, FALSE, TRUE, 0);
 	list = NULL;
 	list = g_list_append(list, text_format);
 	list = g_list_append(list, DEFAULT_TEXT_FORMAT);
@@ -1178,7 +1324,12 @@ create_cpu_tab(GtkWidget *tab_vbox)
 			text_format);
 	g_signal_connect(G_OBJECT(GTK_COMBO(text_format_combo)->entry), "changed",
 			G_CALLBACK(cb_text_format), NULL);
-	
+
+	vbox1 = gtk_vbox_new(FALSE, 0);
+	gtk_box_pack_end(GTK_BOX(hbox), vbox1, FALSE, TRUE, 4);
+	gkrellm_gtk_alert_button(vbox1, NULL, FALSE, FALSE, 4, FALSE,
+			cb_set_alert, NULL);
+
 	vbox1 = gkrellm_gtk_framed_vbox_end(vbox, _("Launch Commands"),
 				4, TRUE, 0,2);
 	vbox1 = gkrellm_gtk_scrolled_vbox(vbox1, NULL,
