@@ -39,9 +39,14 @@ typedef struct
 	GtkWidget		*vbox;
 	GtkWidget		*enable_button;
 	GkrellmChart	*chart;
-	GkrellmChartdata *rb_cd,
-					 *wb_cd;
+	GkrellmChartdata *read_cd,
+					 *write_cd;
 	GkrellmChartconfig *chart_config;
+	GkrellmAlert	*alert;
+	GtkWidget		*alert_config_read_button,
+					*alert_config_write_button;
+	gboolean		alert_uses_read,
+					alert_uses_write;
 	gint			enabled;
 	gint			major,
 					minor,
@@ -118,6 +123,7 @@ new_disk(gchar *name)
 	disk->name = g_strdup(name);
 	disk->launch.command = g_strdup("");
 	disk->launch.tooltip_comment = g_strdup("");
+	disk->alert_uses_read = disk->alert_uses_write = TRUE;
 	return disk;
 	}
 
@@ -181,11 +187,10 @@ add_subdisk(gchar *subdisk_name, gchar *disk_name, gint subdisk)
 static void
 disk_assign_data(DiskMon *disk, guint64 rb, guint64 wb)
 	{
-	if (disk)
-		{
-		disk->rb = rb;
-		disk->wb = wb;
-		}
+	if (!disk)
+		return;
+	disk->rb = rb;
+	disk->wb = wb;
 	if (disk->subdisk == -1)	/* not a subdisk (partition) */
 		{
 		composite_disk->rb += rb;
@@ -327,20 +332,24 @@ static gchar    *text_format,
 				*text_format_locale;
 
 static void
-format_chart_text(DiskMon *disk, gchar *buf, gint size)
+format_disk_data(DiskMon *disk, gchar *src_string, gchar *buf, gint size)
 	{
 	GkrellmChart	*cp;
 	gchar			c, *s;
 	size_t			tbl_size;
 	gint			len, r_blocks, w_blocks, blocks;
 
+	if (!buf || size < 1)
+		return;
 	--size;
 	*buf = '\0';
+	if (!src_string)
+		return;
 	cp = disk->chart;
-	r_blocks = gkrellm_get_current_chartdata(disk->rb_cd);
-	w_blocks = gkrellm_get_current_chartdata(disk->wb_cd);
+	r_blocks = gkrellm_get_current_chartdata(disk->read_cd);
+	w_blocks = gkrellm_get_current_chartdata(disk->write_cd);
 	tbl_size = sizeof(disk_blocks_abbrev) / sizeof(GkrellmSizeAbbrev);
-	for (s = text_format_locale; *s != '\0' && size > 0; ++s)
+	for (s = src_string; *s != '\0' && size > 0; ++s)
 		{
 		len = 1;
 		if (*s == '$' && *(s + 1) != '\0')
@@ -354,6 +363,8 @@ format_chart_text(DiskMon *disk, gchar *buf, gint size)
 				blocks = r_blocks;
 			else if (c == 'w')
 				blocks = w_blocks;
+			else if (c == 'H')
+				len = snprintf(buf, size, "%s", gkrellm_sys_get_host_name());
 			else
 				{
 				*buf = *s;
@@ -383,10 +394,17 @@ draw_disk_extra(DiskMon *disk)
 
 	if (!disk->extra_info)
 		return;
-	format_chart_text(disk, buf, sizeof(buf));
+	format_disk_data(disk, text_format_locale, buf, sizeof(buf));
 	gkrellm_draw_chart_text(disk->chart, style_id, buf);
 	}
 
+static void
+cb_command_process(GkrellmAlert *alert, gchar *src, gchar *dst, gint len,
+			DiskMon *disk)
+	{
+	format_disk_data(disk, src, dst, len);
+	}
+
 static void
 draw_disk_chart(DiskMon *disk)
 	{
@@ -401,6 +419,7 @@ update_disk(void)
 	GList			*list;
 	DiskMon			*disk;
 	GkrellmChart	*cp;
+	gint			bytes;
 
 	if (!sys_handles_composite_reset)
 		{
@@ -420,6 +439,17 @@ update_disk(void)
 			{
 			gkrellm_store_chartdata(cp, 0,
 						(gulong) disk->wb, (gulong) disk->rb);
+			if (disk->alert)
+				{
+				bytes = 0;
+				if (disk->alert_uses_read)
+					bytes += gkrellm_get_current_chartdata(disk->read_cd);
+				if (disk->alert_uses_write)
+					bytes += gkrellm_get_current_chartdata(disk->write_cd);
+				gkrellm_check_alert(disk->alert, bytes);
+				}
+			gkrellm_panel_label_on_top_of_decals(cp->panel,
+						gkrellm_alert_decal_visible(disk->alert));
 			draw_disk_chart(disk);
 			}
 		gkrellm_update_krell(cp->panel, KRELL(cp->panel),
@@ -543,8 +573,8 @@ create_disk_monitor(DiskMon *disk, gint first_create)
 	gkrellm_create_krell(p, gkrellm_krell_panel_piximage(style_id), style);
 
 	gkrellm_chart_create(disk->vbox, mon_disk, cp, &disk->chart_config);
-	disk->wb_cd = gkrellm_add_default_chartdata(cp, _("Write Blocks"));
-	disk->rb_cd = gkrellm_add_default_chartdata(cp, _("Read Blocks"));
+	disk->write_cd = gkrellm_add_default_chartdata(cp, _("Write bytes"));
+	disk->read_cd = gkrellm_add_default_chartdata(cp, _("Read bytes"));
 	gkrellm_set_draw_chart_function(cp, draw_disk_chart, disk);
 
 	gkrellm_chartconfig_fixed_grids_connect(cp->config,
@@ -643,6 +673,36 @@ check_for_new_disks(void)
 		}
 	}
 
+static void
+cb_alert_trigger(GkrellmAlert *alert, DiskMon *disk)
+	{
+	/* Full panel alert, default decal.
+	*/
+	alert->panel = disk->chart->panel;
+	}
+
+
+static void
+create_alert(DiskMon *disk)
+	{
+	static void	cb_alert_config(GkrellmAlert *ap, DiskMon *disk);
+	static void	cb_alert_config_create(GkrellmAlert *ap, GtkWidget *vbox,
+						DiskMon *disk);
+
+	disk->alert = gkrellm_alert_create(NULL, disk->name,
+			_("Bytes per second"),
+			TRUE, FALSE, TRUE,
+			1e9, 1000, 1000, 10000, 0);
+	gkrellm_alert_delay_config(disk->alert, 1, 60 * 180, 0);
+
+	gkrellm_alert_trigger_connect(disk->alert, cb_alert_trigger, disk);
+	gkrellm_alert_config_connect(disk->alert, cb_alert_config, disk);
+	gkrellm_alert_config_create_connect(disk->alert,
+							cb_alert_config_create, disk);
+	gkrellm_alert_command_process_connect(disk->alert,
+				cb_command_process, disk);
+	}
+
 static void
 save_disk_config(FILE *f)
 	{
@@ -666,6 +726,14 @@ save_disk_config(FILE *f)
 						disk->name, disk->launch.tooltip_comment);
 		gkrellm_save_chartconfig(f, disk->chart_config,
 					DISK_CONFIG_KEYWORD, disk->name);
+		if (disk->alert && disk->enabled)
+			{
+			gkrellm_save_alertconfig(f, disk->alert,
+						DISK_CONFIG_KEYWORD, disk->name);
+			fprintf(f, "%s extra_alert_config %s %d %d\n", DISK_CONFIG_KEYWORD,
+						disk->name,
+						disk->alert_uses_read, disk->alert_uses_write);
+			}
 		}
 	fprintf(f, "%s text_format %s\n", DISK_CONFIG_KEYWORD, text_format);
 	}
@@ -675,7 +743,7 @@ load_disk_config(gchar *arg)
 	{
 	DiskMon		*disk;
 	gchar		config[32], item[CFG_BUFSIZE],
-				name[32], command[CFG_BUFSIZE];
+				name[32], item1[CFG_BUFSIZE];
 	gint		major, minor, n, extra, order, subdisk = -1;
 	static gint	config_assign_method;
 
@@ -714,21 +782,38 @@ load_disk_config(gchar *arg)
 			}
 		else if (!strcmp(config, GKRELLM_CHARTCONFIG_KEYWORD))
 			{
-			sscanf(item, "%31s %[^\n]", name, command);
+			sscanf(item, "%31s %[^\n]", name, item1);
 			if ((disk = lookup_disk_by_name(name)) != NULL)
-				gkrellm_load_chartconfig(&disk->chart_config, command, 2);
+				gkrellm_load_chartconfig(&disk->chart_config, item1, 2);
+			}
+		else if (!strcmp(config, GKRELLM_ALERTCONFIG_KEYWORD))
+			{
+			sscanf(item, "%31s %[^\n]", name, item1);
+			if ((disk = lookup_disk_by_name(name)) != NULL)
+				{
+				if (!disk->alert)
+					create_alert(disk);
+				gkrellm_load_alertconfig(&disk->alert, item1);
+				}
+			}
+		else if (!strcmp(config, "extra_alert_config"))
+			{
+			sscanf(item, "%31s %[^\n]", name, item1);
+			if ((disk = lookup_disk_by_name(name)) != NULL)
+				sscanf(item1, "%d %d",
+							&disk->alert_uses_read, &disk->alert_uses_write);
 			}
 		else if (strcmp(config, "launch") == 0)
 			{
-			sscanf(item, "%31s %[^\n]", name, command);
+			sscanf(item, "%31s %[^\n]", name, item1);
 			if ((disk = lookup_disk_by_name(name)) != NULL)
-				disk->launch.command = g_strdup(command);
+				disk->launch.command = g_strdup(item1);
 			}
 		else if (strcmp(config, "tooltip_comment") == 0)
 			{
-			sscanf(item, "%31s %[^\n]", name, command);
+			sscanf(item, "%31s %[^\n]", name, item1);
 			if ((disk = lookup_disk_by_name(name)) != NULL)
-				disk->launch.tooltip_comment = g_strdup(command);
+				disk->launch.tooltip_comment = g_strdup(item1);
 			}
 		}
 	}
@@ -741,14 +826,21 @@ enum
 	NAME_COLUMN,
 	ENABLE_COLUMN,
 	DISK_COLUMN,
+	IMAGE_COLUMN,
 	N_COLUMNS
 	};
 
 static GtkTreeView			*treeview;
 
+static GtkTreeRowReference	*row_reference;
+static GtkTreeSelection		*selection;
+
 static GtkWidget			*launch_vbox,
 							*text_format_combo;
 
+static GtkWidget			*alert_button;
+
+
 
 static GtkTreeModel *
 create_model(void)
@@ -761,7 +853,8 @@ create_model(void)
 	tree = gtk_tree_store_new(N_COLUMNS,
 				G_TYPE_STRING,
                 G_TYPE_BOOLEAN,
-				G_TYPE_POINTER);
+				G_TYPE_POINTER,
+				GDK_TYPE_PIXBUF);
 	for (list = disk_mon_list; list; )
 		{
 		disk = (DiskMon *) list->data;
@@ -773,11 +866,16 @@ create_model(void)
 				DISK_COLUMN, disk,
 				-1);
 		else
+			{
 			gtk_tree_store_set(tree, &iter,
 					NAME_COLUMN, disk->name,
 					ENABLE_COLUMN, disk->enabled,
 					DISK_COLUMN, disk,
 					-1);
+			if (disk->alert)
+				gtk_tree_store_set(tree, &iter,
+						IMAGE_COLUMN, gkrellm_alert_pixbuf(), -1);
+			}
 		for (clist = list->next; clist; clist = clist->next)
 			{
 			disk = (DiskMon *) clist->data;
@@ -789,6 +887,9 @@ create_model(void)
 					ENABLE_COLUMN, disk->enabled,
 					DISK_COLUMN, disk,
 					-1);
+			if (disk->alert)
+				gtk_tree_store_set(tree, &citer,
+						IMAGE_COLUMN, gkrellm_alert_pixbuf(), -1);
 			}
 		list = clist;
 		}
@@ -818,6 +919,140 @@ add_launch_entry(GtkWidget *vbox, DiskMon *disk)
 	gtk_widget_show_all(disk->launch_table);
     }
 
+static void
+change_row_reference(GtkTreeModel *model, GtkTreePath *path)
+	{
+	gtk_tree_row_reference_free(row_reference);
+	if (model && path)
+		row_reference = gtk_tree_row_reference_new(model, path);
+	else
+		row_reference = NULL;
+	}
+
+static void
+cb_alert_config(GkrellmAlert *ap, DiskMon *disk)
+	{
+	GtkTreeModel	*model;
+	GtkTreeIter		iter, citer;
+	DiskMon			*disk_test;
+	GdkPixbuf		*pixbuf;
+	gboolean		valid;
+
+	disk->alert_uses_read =
+				GTK_TOGGLE_BUTTON(disk->alert_config_read_button)->active;
+	disk->alert_uses_write =
+				GTK_TOGGLE_BUTTON(disk->alert_config_write_button)->active;
+	if (!gkrellm_config_window_shown())
+		return;
+	model = gtk_tree_view_get_model(treeview);
+	pixbuf = ap->activated ? gkrellm_alert_pixbuf() : NULL;
+	valid = gtk_tree_model_get_iter_first(model, &iter);
+	while (valid)
+		{
+		gtk_tree_model_get(model, &iter, DISK_COLUMN, &disk_test, -1);
+		if (disk == disk_test)
+			{
+			gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
+						IMAGE_COLUMN, pixbuf, -1);
+			return;
+			}
+		if (gtk_tree_model_iter_children(model, &citer, &iter))
+			do
+				{
+				gtk_tree_model_get(model, &citer, DISK_COLUMN, &disk_test, -1);
+				if (disk == disk_test)
+					{
+					gtk_tree_store_set(GTK_TREE_STORE(model), &citer,
+								IMAGE_COLUMN, pixbuf, -1);
+					return;
+					}
+				}
+			while (gtk_tree_model_iter_next(model, &citer));
+
+		valid = gtk_tree_model_iter_next(model, &iter);
+		}
+	}
+
+static void
+cb_alert_config_button(GtkWidget *button, DiskMon *disk)
+	{
+	gboolean	read, write;
+
+	read = GTK_TOGGLE_BUTTON(disk->alert_config_read_button)->active;
+	write = GTK_TOGGLE_BUTTON(disk->alert_config_write_button)->active;
+	if (!read && !write)
+		{
+		gtk_toggle_button_set_active(
+				GTK_TOGGLE_BUTTON(disk->alert_config_read_button), TRUE);
+		gtk_toggle_button_set_active(
+				GTK_TOGGLE_BUTTON(disk->alert_config_write_button), TRUE);
+		}
+	}
+
+static void
+cb_alert_config_create(GkrellmAlert *ap, GtkWidget *vbox, DiskMon *disk)
+	{
+    gkrellm_gtk_check_button_connected(vbox, &disk->alert_config_read_button,
+				disk->alert_uses_read, FALSE, FALSE, 2,
+				cb_alert_config_button, disk, _("Read bytes"));
+    gkrellm_gtk_check_button_connected(vbox, &disk->alert_config_write_button,
+				disk->alert_uses_write, FALSE, FALSE, 2,
+				cb_alert_config_button, disk, _("Write bytes"));
+	}
+
+static void
+cb_set_alert(GtkWidget *button, gpointer data)
+	{
+	GtkTreeModel	*model;
+	GtkTreePath		*path;
+	GtkTreeIter		iter;
+	DiskMon			*disk;
+
+	if (!row_reference)
+		return;
+	model = gtk_tree_view_get_model(treeview);
+	path = gtk_tree_row_reference_get_path(row_reference);
+	gtk_tree_model_get_iter(model, &iter, path);
+	gtk_tree_model_get(model, &iter, DISK_COLUMN, &disk, -1);
+
+	if (!disk->alert)
+		create_alert(disk);
+	gkrellm_alert_config_window(&disk->alert);
+	}
+
+
+static void
+cb_tree_selection_changed(GtkTreeSelection *selection, gpointer data)
+	{
+	GtkTreeIter     iter;
+	GtkTreeModel    *model;
+	GtkTreePath     *path;
+	DiskMon			*disk;
+	gint            *indices, depth;
+
+	if (!gtk_tree_selection_get_selected(selection, &model, &iter))
+		{
+		gtk_widget_set_sensitive(alert_button, FALSE);
+		return;
+		}
+	path = gtk_tree_model_get_path(model, &iter);
+	indices = gtk_tree_path_get_indices(path);
+	depth = gtk_tree_path_get_depth(path);
+// printf("selection: indices=[%d,%d]:%d, path=%s\n",
+//          indices[0], indices[1], gtk_tree_path_get_depth(path),
+//          gtk_tree_path_to_string(path));
+	change_row_reference(model, path);
+
+	gtk_tree_model_get(model, &iter,
+			DISK_COLUMN, &disk, -1);
+
+	if (!disk->enabled || (depth == 1 && indices[0] == 0))
+		gtk_widget_set_sensitive(alert_button, FALSE);
+	else
+		gtk_widget_set_sensitive(alert_button, TRUE);
+	gtk_tree_path_free(path);
+	}
+
 static void
 cb_enable(GtkCellRendererText *cell, gchar *path_string, gpointer data)
 	{
@@ -841,11 +1076,15 @@ cb_enable(GtkCellRendererText *cell, gchar *path_string, gpointer data)
 				ENABLE_COLUMN, !enabled, 
 				-1);
 	if (enabled)
+		{
 		destroy_disk_monitor(disk);
+		gtk_widget_set_sensitive(alert_button, FALSE);
+		}
 	else
 		{
 		create_disk_monitor(disk, TRUE);
 		add_launch_entry(launch_vbox, disk);
+		gtk_widget_set_sensitive(alert_button, TRUE);
 		}
 	disk->enabled = !enabled;
 	enabled = FALSE;
@@ -881,6 +1120,8 @@ N_("\t$M    maximum chart value\n"),
 N_("\t$T    total read blocks + write blocks\n"),
 N_("\t$r    read blocks\n"),
 N_("\t$w    write blocks\n"),
+"\n",
+N_("Substitution variables may be used in alert commands.\n")
 };
 
 static void
@@ -906,6 +1147,7 @@ create_disk_tab(GtkWidget *tab_vbox)
 
 /* -- Options tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Options"));
+
 	scrolled = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
 				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -930,9 +1172,24 @@ create_disk_tab(GtkWidget *tab_vbox)
 				"text", NAME_COLUMN,
 				NULL);
 
+	renderer = gtk_cell_renderer_pixbuf_new();
+	gtk_tree_view_insert_column_with_attributes(treeview, -1, "",
+				renderer,
+				"pixbuf", IMAGE_COLUMN,
+				NULL);
+
 	gtk_container_add(GTK_CONTAINER(scrolled), GTK_WIDGET(treeview));
 	g_object_unref(G_OBJECT(model));
+	selection = gtk_tree_view_get_selection(treeview);
+	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
+	g_signal_connect(G_OBJECT(selection), "changed",
+				G_CALLBACK(cb_tree_selection_changed), NULL);
 
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 3);
+	gkrellm_gtk_alert_button(hbox, &alert_button, FALSE, FALSE, 4, FALSE,
+				cb_set_alert, NULL);
+	gtk_widget_set_sensitive(alert_button, FALSE);
 
 /* -- Setup tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Setup"));
