@@ -63,6 +63,8 @@ typedef struct
 	GkrellmLauncher	launch_mount,
 				launch_umount;
 
+	GkrellmAlert *alert;
+
 	gboolean	secondary,
 				show_if_mounted,
 				is_mounted,
@@ -331,7 +333,7 @@ static GkrellmSizeAbbrev	fs_binary_abbrev[] =
 	};
 
 static gint
-format_fs_data(FSmon *fs, gchar *buf, gint size)
+format_fs_data(FSmon *fs, gchar *src_string, gchar *buf, gint size)
 	{
 	gulong		b, u, f;
 	gint		len;
@@ -341,8 +343,12 @@ format_fs_data(FSmon *fs, gchar *buf, gint size)
 	GkrellmSizeAbbrev *tbl;
 	size_t		tbl_size;
 
+	if (!buf || size < 1)
+		return -1;
 	--size;
 	*buf = '\0';
+	if (!src_string)
+		return -1;
 	b = fs->blocks;
 	u = fs->blocks - fs->bavail;
 	f = fs->bavail;
@@ -358,14 +364,19 @@ format_fs_data(FSmon *fs, gchar *buf, gint size)
 	format_size_abbrev(fbuf, sizeof(fbuf), (gfloat)f * bsize, tbl, tbl_size);
 
 	f = b - u;
-	for (s = data_format_locale; *s != '\0' && size > 0; ++s)
+	for (s = src_string; *s != '\0' && size > 0; ++s)
 		{
 		len = 1;
 		if (*s == '$' && *(s + 1) != '\0')
 			{
 			switch(*(s + 1))
 				{
+				case 'D':
+					if (fs->mount.directory)
+						len = snprintf(buf, size, "%s", fs->mount.directory);
+					break;
 				case 'l':
+				case 'L':
 					len = snprintf(buf, size, "%s", fs->label_shadow);
 					break;
 				case 't':
@@ -387,6 +398,10 @@ format_fs_data(FSmon *fs, gchar *buf, gint size)
 						len = snprintf(buf, size, "%ld%%",
 							100 * (f >> 8) / (b >> 8));
 					break;
+				case 'H':
+					len = snprintf(buf, size, "%s",
+									gkrellm_sys_get_host_name());
+					break;
 				default:
 					*buf = *s;
 					if (size > 1)
@@ -430,7 +445,7 @@ draw_fs_text_decal(FSmon *fs, gint value)
 		ts_save = d->text_style;
 		d->text_style = *gkrellm_meter_alt_textstyle(style_id);
 
-		changed = format_fs_data(fs, buf, sizeof(buf));
+		changed = format_fs_data(fs, data_format_locale, buf, sizeof(buf));
 		w = gkrellm_gdk_string_width(d->text_style.font, buf);
 		if (w  > d->w)
 			{
@@ -450,6 +465,12 @@ draw_fs_text_decal(FSmon *fs, gint value)
 	return w;
 	}
 
+static void
+cb_command_process(GkrellmAlert *alert, gchar *src, gchar *dst, gint len,
+			FSmon *fs)
+	{
+	format_fs_data(fs, src, dst, len);
+	}
 
 static gpointer
 close_cdrom_thread(void *device)
@@ -663,12 +684,14 @@ animate_eject_button(FSmon *fs, gboolean force_close)
 static void
 update_fs(void)
 	{
-	FSmon		*fs;
-	GkrellmPanel *p;
-	GList		*list;
-	gulong		l;
-	gint		index, w_scroll, w;
-	gboolean	fs_check, nfs_check, force_check, force_draw, mounting_enabled;
+	FSmon			*fs;
+	GkrellmPanel	*p;
+	GkrellmKrell	*k;
+	GList			*list;
+	gulong			l, used;
+	gint			full_scale, index, w_scroll, w;
+	gboolean		fs_check, nfs_check, force_check, force_draw,
+					mounting_enabled;
 
 	if (!fs_mon_list)
 		return;
@@ -715,7 +738,8 @@ printf("fs update %d nfs %d force %d\n", fs_check, nfs_check, force_fs_check);
 		{
 		fs = (FSmon *) list->data;
 		p  = fs->panel;
-		KRELL(p)->previous = 0;
+		k = KRELL(p);
+		k->previous = 0;
 		mounting_enabled = FS_MOUNTING_ENABLED(fs);
 		if (fs_is_mounted(fs))
 			{
@@ -751,15 +775,21 @@ printf("fs update %d nfs %d force %d\n", fs_check, nfs_check, force_fs_check);
 					}
 				fs->krell_factor = fs->blocks > 2097152 ? 1024 : 1;
 				}			
-			KRELL(p)->full_scale = fs->blocks / fs->krell_factor;
-			l = (fs->blocks - fs->bavail) / fs->krell_factor;
+			full_scale = (gint) (fs->blocks / fs->krell_factor);
+			gkrellm_set_krell_full_scale(k, full_scale, 1);
+
+			used = l = (fs->blocks - fs->bavail) / fs->krell_factor;
 			if (   (fs_in_motion && fs->label_is_data && x_moved)
 				|| fs->mouse_entered
 			   )
 				l = 0;
 
 			if (!fs->busy)
-				gkrellm_update_krell(p, KRELL(p), l);
+				{
+				gkrellm_update_krell(p, k, l);
+				gkrellm_check_alert(fs->alert,
+							100.0 * (gfloat) used / (gfloat) full_scale);
+				}
 			else
 				force_draw = TRUE;
 
@@ -768,8 +798,9 @@ printf("fs update %d nfs %d force %d\n", fs_check, nfs_check, force_fs_check);
 			if (fs->eject_decal)
 				force_draw |= animate_eject_button(fs, mounting_supported);
 			}
-		else
+		else	/* not mounted */
 			{
+			gkrellm_reset_alert(fs->alert);
 			if (mounting_enabled)
 				{	/* Blink it while pipe is open.	*/
 				if (   fs->launch_mount.pipe
@@ -782,8 +813,8 @@ printf("fs update %d nfs %d force %d\n", fs_check, nfs_check, force_fs_check);
 				}
 			else
 				gkrellm_set_decal_button_index(fs->md_button, D_MISC_LED0);
-			KRELL(p)->full_scale = 100;		/* Arbitrary > 0 */
-			gkrellm_update_krell(p, KRELL(p), 0);
+			gkrellm_set_krell_full_scale(k, 100, 1);	/* Arbitrary > 0 */
+			gkrellm_update_krell(p, k, 0);
 			if (!secondary_monitors_shown && fs->secondary)
 				gkrellm_panel_hide(fs->panel);
 			if (fs->eject_decal)
@@ -980,7 +1011,7 @@ cb_panel_motion(GtkWidget *widget, GdkEventButton *ev)
 	font = gkrellm_meter_alt_textstyle(style_id)->font;
 
 	d = fs_in_motion->text_decal;
-	format_fs_data(fs_in_motion, buf, sizeof(buf));
+	format_fs_data(fs_in_motion, data_format_locale, buf, sizeof(buf));
 	w = gkrellm_gdk_string_width(font, buf);
 	if (w > d->w)
 		{
@@ -1153,6 +1184,7 @@ free_fsmon_strings(FSmon *fs)
 static void
 destroy_fs_monitor(FSmon *fs)
 	{
+	gkrellm_reset_alert(fs->alert);
 	free_fsmon_strings(fs);
 	gkrellm_panel_destroy(fs->panel);
 	g_free(fs);
@@ -1190,17 +1222,41 @@ create_fs(GtkWidget *vbox, gint first_create)
 
 #define	FS_CONFIG_KEYWORD	"fs"
 
+static void
+cb_alert_trigger(GkrellmAlert *alert, FSmon *fs)
+	{
+	/* Full panel alert, default decal.
+	*/
+	alert->panel = fs->panel;
+	}
+
+static void
+create_alert(FSmon *fs)
+	{
+	static void cb_alert_config(GkrellmAlert *ap, FSmon *fs);
+
+	fs->alert = gkrellm_alert_create(NULL, fs->label,
+				_("Percent Usage"),
+				TRUE, FALSE, TRUE,
+				100, 10, 1, 10, 0);
+	gkrellm_alert_trigger_connect(fs->alert, cb_alert_trigger, fs);
+	gkrellm_alert_config_connect(fs->alert, cb_alert_config, fs);
+	gkrellm_alert_command_process_connect(fs->alert, cb_command_process, fs);
+	}
+
 static void
 save_fs_config(FILE *f)
 	{
 	GList	*list;
 	FSmon	*fs;
+	gchar	quoted_label[64];
 
 	for (list = fs_mon_list; list; list = list->next)
 		{
 		fs = (FSmon *) list->data;
-		fprintf(f, "%s \"%s\" %s %d %d %d %d %d\n", FS_CONFIG_KEYWORD,
-				fs->label, fs->mount.directory,
+		snprintf(quoted_label, sizeof(quoted_label), "\"%s\"", fs->label);
+		fprintf(f, "%s %s %s %d %d %d %d %d\n", FS_CONFIG_KEYWORD,
+				quoted_label, fs->mount.directory,
 				fs->fstab_mounting, fs->secondary,
 				fs->show_if_mounted, fs->label_is_data, fs->ejectable);
 		if (*(fs->launch_mount.command))
@@ -1212,6 +1268,9 @@ save_fs_config(FILE *f)
 		if (*(fs->eject_device))
 			fprintf(f, "%s eject_device %s\n", FS_CONFIG_KEYWORD,
 					fs->eject_device);
+		if (fs->alert)
+			gkrellm_save_alertconfig(f, fs->alert,
+					FS_CONFIG_KEYWORD, quoted_label);
 		}
 	if (!_GK.client_mode)
 		{
@@ -1256,6 +1315,23 @@ fix_fstab_mountable_changed(FSmon *fs)
 	return FALSE;
 	}
 
+static FSmon *
+lookup_fs(gchar *name)
+	{
+	GList	*list;
+	FSmon	*fs;
+
+	if (!name)
+		return NULL;
+	for (list = fs_mon_list; list; list = list->next)
+		{
+		fs = (FSmon *) list->data;
+		if (!strcmp(fs->label, name))
+			return fs;
+		}
+	return NULL;
+	}
+
 static void
 load_fs_config(gchar *arg)
 	{
@@ -1263,6 +1339,7 @@ load_fs_config(gchar *arg)
 	FSmon			*fs;
 	gchar			*label;
 	gchar			config[32], item[CFG_BUFSIZE], dir[CFG_BUFSIZE];
+	gchar			name[64], item1[CFG_BUFSIZE];
 	gint 			n;
 
 	if ((n = sscanf(arg, "%31s %[^\n]", config, item)) != 2)
@@ -1295,6 +1372,17 @@ load_fs_config(gchar *arg)
 		if (fs_prev->ejectable)
 			gkrellm_dup_string(&fs_prev->eject_device, item);
 		}
+	else if (!strcmp(config, GKRELLM_ALERTCONFIG_KEYWORD))
+		{
+		if (   sscanf(item, "\"%64[^\"]\" %[^\n]", name, item1) == 2
+			&& (fs = lookup_fs(name)) != NULL
+		   )
+			{
+			if (!fs->alert)
+				create_alert(fs);
+			gkrellm_load_alertconfig(&fs->alert, item1);
+			}
+		}
 	else
 		{
 		if ((label = cut_quoted_string(arg, &arg)) != NULL)
@@ -1341,7 +1429,11 @@ enum
 	UMOUNT_COMMAND_COLUMN,
 	EJECTABLE_COLUMN,
 	DEVICE_COLUMN,
+	FSMON_COLUMN,
+	ALERT_COLUMN,
+	SHOW_DATA_COLUMN,
 	VISIBLE_COLUMN,
+	IMAGE_COLUMN,
 	N_COLUMNS
 	};
 
@@ -1360,6 +1452,7 @@ static GtkWidget
 				*secondary_button,
 				*show_button;
 
+static GtkWidget	*alert_button;
 
 static GtkWidget	*data_format_combo;
 
@@ -1378,8 +1471,15 @@ set_tree_store_model_data(GtkTreeStore *tree, GtkTreeIter *iter, FSmon *fs)
 			UMOUNT_COMMAND_COLUMN, fs->launch_umount.command,
 			EJECTABLE_COLUMN, fs->ejectable,
 			DEVICE_COLUMN, fs->eject_device,
+			FSMON_COLUMN, fs,
+			ALERT_COLUMN, fs->alert,
+			SHOW_DATA_COLUMN, fs->label_is_data,
 			VISIBLE_COLUMN, TRUE,
 			-1);
+	if (fs->alert)
+		gtk_tree_store_set(tree, iter,
+				IMAGE_COLUMN, gkrellm_alert_pixbuf(),
+				-1);
 	}
 
 static GtkTreeModel *
@@ -1395,7 +1495,9 @@ create_model(void)
 				G_TYPE_BOOLEAN, G_TYPE_BOOLEAN,
 				G_TYPE_STRING, G_TYPE_STRING,
 				G_TYPE_BOOLEAN, G_TYPE_STRING,
-				G_TYPE_BOOLEAN);
+				G_TYPE_POINTER, G_TYPE_POINTER,
+				G_TYPE_BOOLEAN,
+				G_TYPE_BOOLEAN, GDK_TYPE_PIXBUF);
 
 	gtk_tree_store_append(tree, &iter, NULL);
 	gtk_tree_store_set(tree, &iter,
@@ -1437,6 +1539,75 @@ change_row_reference(GtkTreeModel *model, GtkTreePath *path)
 		row_reference = NULL;
 	}
 
+static void
+cb_set_alert(GtkWidget *button, gpointer data)
+	{
+	GtkTreeModel	*model;
+	GtkTreePath		*path;
+	GtkTreeIter		iter;
+	FSmon			*fs;
+
+	if (!row_reference)
+		return;
+	model = gtk_tree_view_get_model(treeview);
+	path = gtk_tree_row_reference_get_path(row_reference);
+	gtk_tree_model_get_iter(model, &iter, path);
+	gtk_tree_model_get(model, &iter, FSMON_COLUMN, &fs, -1);
+
+	if (!fs->alert)
+		create_alert(fs);
+	gkrellm_alert_config_window(&fs->alert);
+	gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
+						ALERT_COLUMN, fs->alert, -1);
+	}
+
+static gboolean
+get_child_iter(GtkTreeModel *model, gchar *parent_node, GtkTreeIter *citer)
+	{
+	GtkTreePath     *path;
+	GtkTreeIter     iter;
+
+	path = gtk_tree_path_new_from_string(parent_node);
+	gtk_tree_model_get_iter(model, &iter, path);
+	gtk_tree_path_free(path);
+	return gtk_tree_model_iter_children(model, citer, &iter);
+	}
+
+  /* Callback for a created or destroyed alert.  Find the sensor in the model
+  |  and set the IMAGE_COLUMN.
+  */
+static void
+cb_alert_config(GkrellmAlert *ap, FSmon *fs)
+	{
+	GtkTreeModel    *model;
+	GtkTreeIter     iter;
+	FSmon			*fs_test;
+	GdkPixbuf       *pixbuf;
+	gchar           node[2];
+	gint            i;
+
+	if (!gkrellm_config_window_shown())
+		return;
+	model = gtk_tree_view_get_model(treeview);
+	pixbuf = ap->activated ? gkrellm_alert_pixbuf() : NULL;
+	for (i = 0; i < 2; ++i)
+		{
+		node[0] = '0' + i;      /* toplevel Primary or Secondary node */
+		node[1] = '\0';
+		if (get_child_iter(model, node, &iter))
+			do
+				{
+				gtk_tree_model_get(model, &iter, FSMON_COLUMN, &fs_test, -1);
+				if (fs != fs_test)
+					continue;
+				gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
+							IMAGE_COLUMN, pixbuf, -1);
+				return;
+				}
+			while (gtk_tree_model_iter_next(model, &iter));
+		}
+	}
+
   /* Watch what is going into the directory combo entry, compare it to
   |  fstab entries and accordingly set sensitivity of the mounting_button.
   */
@@ -1578,6 +1749,8 @@ fs_new_from_model(GtkTreeModel *model, GtkTreeIter *iter)
 				UMOUNT_COMMAND_COLUMN, &fs->launch_umount.command,
 				EJECTABLE_COLUMN, &fs->ejectable,
 				DEVICE_COLUMN, &fs->eject_device,
+				ALERT_COLUMN, &fs->alert,
+				SHOW_DATA_COLUMN, &fs->label_is_data,
 				-1);
 	gkrellm_locale_dup_string(&fs->label, label, &fs->label_shadow);
 	g_free(label);
@@ -1596,6 +1769,7 @@ cb_tree_selection_changed(GtkTreeSelection *selection, gpointer data)
 	if (!gtk_tree_selection_get_selected(selection, &model, &iter))
 		{
 		reset_entries(FALSE);
+		gtk_widget_set_sensitive(alert_button, FALSE);
 		return;
 		}
 	path = gtk_tree_model_get_path(model, &iter);
@@ -1611,8 +1785,10 @@ cb_tree_selection_changed(GtkTreeSelection *selection, gpointer data)
 	if (depth == 1)
 		{
 		reset_entries(secondary);
+		gtk_widget_set_sensitive(alert_button, FALSE);
 		return;
 		}
+	gtk_widget_set_sensitive(alert_button, TRUE);
 	fs = fs_new_from_model(model, &iter);
 	if (!secondary)
 		fs->show_if_mounted = FALSE;	/* in case dragged secondary->primary*/
@@ -1692,6 +1868,10 @@ N_("At least one primary fs monitor must exist to click on in order to show\n"
 			fs->show_if_mounted = FALSE;
 			fs_mon_list = g_list_append(fs_mon_list, fs);
 			create_fs_monitor(fs_main_vbox, fs, n_fs_monitors, TRUE);
+			gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
+						FSMON_COLUMN, fs, -1);
+			gkrellm_alert_trigger_connect(fs->alert, cb_alert_trigger, fs);
+			gkrellm_alert_config_connect(fs->alert, cb_alert_config, fs);
 			}
 		while (gtk_tree_model_iter_next(model, &iter));
 		}
@@ -1704,6 +1884,10 @@ N_("At least one primary fs monitor must exist to click on in order to show\n"
 			fs->secondary = TRUE;
 			fs_mon_list = g_list_append(fs_mon_list, fs);
 			create_fs_monitor(fs_secondary_vbox, fs, n_fs_monitors, TRUE);
+			gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
+						FSMON_COLUMN, fs, -1);
+			gkrellm_alert_trigger_connect(fs->alert, cb_alert_trigger, fs);
+			gkrellm_alert_config_connect(fs->alert, cb_alert_config, fs);
 			}
 		while (gtk_tree_model_iter_next(model, &iter));
 		}
@@ -1833,12 +2017,20 @@ cb_delete(GtkWidget *widget)
 	GtkTreeModel	*model;
 	GtkTreePath		*path;
 	GtkTreeIter		iter;
+	FSmon			*fs;
 
 	if (!row_reference)
 		return;
 	model = gtk_tree_view_get_model(treeview);
 	path = gtk_tree_row_reference_get_path(row_reference);
 	gtk_tree_model_get_iter(model, &iter, path);
+
+	gtk_tree_model_get(model, &iter, FSMON_COLUMN, &fs, -1);
+	gkrellm_reset_alert(fs->alert);
+	if (fs->alert)
+		g_free(fs->alert);
+	fs->alert = NULL;
+
 	gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
 	reset_entries(FALSE);
 	sync_fs_panels();
@@ -2064,6 +2256,10 @@ create_fs_panels_page(GtkWidget *vbox)
 	gkrellm_gtk_button_connected(vbox1, NULL, FALSE, FALSE, 4,
 			cb_delete, NULL, GTK_STOCK_DELETE);
 
+	gkrellm_gtk_alert_button(vbox1, &alert_button, FALSE, FALSE, 4, FALSE,
+			cb_set_alert, NULL);
+	gtk_widget_set_sensitive(alert_button, FALSE);
+
 	scrolled = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
 			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -2091,7 +2287,13 @@ create_fs_panels_page(GtkWidget *vbox)
 				"text", MOUNT_POINT_COLUMN,
 //				"visible", VISIBLE_COLUMN,
 				NULL);
-	
+
+	renderer = gtk_cell_renderer_pixbuf_new();
+	gtk_tree_view_insert_column_with_attributes(treeview, -1, "",
+				renderer,
+				"pixbuf", IMAGE_COLUMN,
+				NULL);
+
 	gtk_container_add(GTK_CONTAINER(scrolled), GTK_WIDGET(treeview));
 
 	selection = gtk_tree_view_get_selection(treeview);
@@ -2166,6 +2368,13 @@ N_("\t$f    free space\n"),
 N_("\t$U    used %,\n"),
 N_("\t$F    free %\n"),
 N_("\t$l    the panel label\n"),
+#if !defined(WIN32)
+N_("\t$D    the mount point\n"),
+#else
+N_("\t$D    the disk\n"),
+#endif
+"\n",
+N_("Substitution variables may be used in alert commands.\n"),
 "\n",
 N_("<h>Mouse Button Actions:\n"),
 N_("<b>\tLeft "),
