@@ -45,6 +45,8 @@ struct	GkrellmConfig
 	gint		up_minutes;
 	gint		base_uptime;
 	time_t		start_time;
+	time_t		time_now;
+
 	gint		cpu_sys_activity;
 	gint		sensor_temp_files;
 
@@ -68,6 +70,9 @@ struct	GkrellmConfig
 	gchar		*server_hostname;
 	gchar		*server_sysname;
 	gboolean	client_mode;
+	gint		client_server_reconnect_timeout;
+	gint		client_server_io_timeout;
+	time_t		client_server_read_time;
 
 	gboolean	force_host_config;
 	gboolean	found_host_config;
@@ -272,7 +277,6 @@ struct	GkrellmConfig
 extern struct GkrellmConfig  _GK;
 
 extern struct tm	gkrellm_current_tm;
-extern time_t		time_now;
 
 extern gint			n_smp_cpus;
 extern gint			gkrellm_w_display,
@@ -452,6 +456,7 @@ void	gkrellm_winop_state_below(gboolean);
 
 /* client */
 gboolean	gkrellm_client_mode_connect(void);
+void		gkrellm_client_mode_disconnect(void);
 struct tm	*gkrellm_client_server_time(void);
 gint		gkrellm_client_server_connect_state(void);
 void		gkrellm_client_mode_connect_thread(void);
