@@ -142,6 +142,7 @@ gboolean	gkrellm_panel_enable_visibility(GkrellmPanel *p, gboolean,
 gboolean	gkrellm_is_panel_visible(GkrellmPanel *);
 void		gkrellm_panel_keep_lists(GkrellmPanel *, gboolean);
 void		gkrellm_draw_panel_label(GkrellmPanel *);
+void		gkrellm_panel_label_on_top_of_decals(GkrellmPanel *, gboolean);
 void		gkrellm_draw_panel_layers(GkrellmPanel *);
 void		gkrellm_draw_panel_layers_force(GkrellmPanel *);
 void		gkrellm_panel_bg_piximage_override(GkrellmPanel *,
@@ -316,19 +317,26 @@ void		gkrellm_spacers_set_types(GkrellmMonitor *mon, gint top, gint bot);
 GkrellmAlert *gkrellm_alert_create(GkrellmPanel *, gchar *, gchar *,
 						gboolean, gboolean, gboolean,
 						gfloat, gfloat, gfloat, gfloat, gint);
+void		gkrellm_alert_dup(GkrellmAlert **a_dst, GkrellmAlert *a_src);
 void		gkrellm_alert_destroy(GkrellmAlert **);
 void		gkrellm_check_alert(GkrellmAlert *, gfloat);
 void		gkrellm_reset_alert(GkrellmAlert *);
 void		gkrellm_reset_panel_alerts(GkrellmPanel *);
 void		gkrellm_freeze_alert(GkrellmAlert *);
 void		gkrellm_thaw_alert(GkrellmAlert *);
+gboolean	gkrellm_alert_is_activated(GkrellmAlert *alert);
 void		gkrellm_alert_trigger_connect(GkrellmAlert *, void (*func)(),
 						gpointer);
 void		gkrellm_alert_stop_connect(GkrellmAlert *, void (*func)(),
 						gpointer);
 void		gkrellm_alert_config_connect(GkrellmAlert *, void (*func)(),
 						gpointer);
+void		gkrellm_alert_config_create_connect(GkrellmAlert *alert,
+						void (*func)(), gpointer);
+void		gkrellm_alert_command_process_connect(GkrellmAlert *alert,
+						void (*func)(), gpointer data);
 void		gkrellm_render_default_alert_decal(GkrellmAlert *);
+gboolean	gkrellm_alert_decal_visible(GkrellmAlert *alert);
 void		gkrellm_alert_config_window(GkrellmAlert **);
 void		gkrellm_alert_window_destroy(GkrellmAlert **);
 void		gkrellm_save_alertconfig(FILE *, GkrellmAlert *, gchar *, gchar *);
@@ -336,6 +344,9 @@ void		gkrellm_load_alertconfig(GkrellmAlert **, gchar *);
 void		gkrellm_alert_set_triggers(GkrellmAlert *alert,
 						gfloat high_alarm, gfloat high_warn,
 						gfloat low_warn, gfloat low_alarm);
+void		gkrellm_alert_set_delay(GkrellmAlert *alert, gint delay);
+void		gkrellm_alert_delay_config(GkrellmAlert *alert, gint step,
+						gint high, gint low);
 
 
   /* GKrellM Styles and Textstyles
@@ -416,6 +427,9 @@ void		gkrellm_gtk_check_button_connected(GtkWidget *, GtkWidget **,
 						gpointer, gchar *);
 void		gkrellm_gtk_button_connected(GtkWidget *, GtkWidget **, gboolean,
 						gboolean, gint, void (*func)(), gpointer, gchar *);
+void		gkrellm_gtk_alert_button(GtkWidget *box, GtkWidget **button,
+						gboolean expand, gboolean fill, gint pad,
+						gboolean pack_start, void (*cb_func)(), gpointer data);
 GtkWidget	*gkrellm_gtk_scrolled_vbox(GtkWidget *, GtkWidget **,
 						GtkPolicyType, GtkPolicyType);
 GtkWidget	*gkrellm_gtk_framed_vbox(GtkWidget *, gchar *, gint, gboolean,
