@@ -89,7 +89,7 @@
 */
 #define	GKRELLM_VERSION_MAJOR	2
 #define	GKRELLM_VERSION_MINOR	1
-#define	GKRELLM_VERSION_REV		7
+#define	GKRELLM_VERSION_REV		8
 #define	GKRELLM_EXTRAVERSION	""
 
 #define GKRELLM_CHECK_VERSION(major,minor,rev)    \
@@ -458,6 +458,7 @@ typedef struct
 	gboolean	modified;		/* True if decal/krell modified.	*/
 	gboolean	keep_lists;
 	gboolean	shown;
+	gboolean	label_on_top_of_decals;
 	gboolean	bg_piximage_override;
 	gboolean	button_signals_connected;
 	gint		id_press,		/* Signals if buttons in this panel	*/
@@ -702,6 +703,8 @@ typedef struct
 				alarm_on;
 	gfloat		warn_limit,
 				alarm_limit;
+	gint		warn_delay,
+				alarm_delay;
 	GtkWidget	*warn_limit_spin_button,
 				*alarm_limit_spin_button;
 	}
@@ -750,6 +753,7 @@ typedef struct
 				alarm_repeat,
 				voice_repeat_set,
 				voice_repeat;
+	gint		delay;
 
 	void		(*cb_trigger)();
 	gpointer	cb_trigger_data;
@@ -757,6 +761,10 @@ typedef struct
 	gpointer	cb_stop_data;
 	void		(*cb_config)();
 	gpointer	cb_config_data;
+	void		(*cb_config_create)();
+	gpointer	cb_config_create_data;
+	void		(*cb_command_process)();
+	gpointer	cb_command_process_data;
 
 	GtkWidget	*config_window,
 				*warn_command_entry,
@@ -764,14 +772,20 @@ typedef struct
 				*voice_string_entry,
 				*warn_repeat_spin_button,
 				*alarm_repeat_spin_button,
-				*voice_repeat_spin_button;
+				*voice_repeat_spin_button,
+				*delay_spin_button,
+				*delete_button;
 
-	gfloat		max_high,		/* Adjustment values */
+	gfloat		max_high,		/* limit adjustment values */
 				min_low,
 				step0,
 				step1;
 	gint		digits;
 
+	gint		delay_high,		/* delay adjustment values */
+				delay_low,
+				delay_step;
+
 	GkrellmTrigger low,
 				high;
 	}
