@@ -466,6 +466,36 @@ gkrellm_gtk_button_connected(GtkWidget *box, GtkWidget **button,
 		*button = b;
 	}
 
+void
+gkrellm_gtk_alert_button(GtkWidget *box, GtkWidget **button,
+			gboolean expand, gboolean fill, gint pad, gboolean pack_start,
+			void (*cb_func)(), gpointer data)
+	{
+	GtkWidget	*b, *hbox;
+	GtkWidget	*image, *label;
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(hbox), 2);
+	image = gtk_image_new_from_pixbuf(gkrellm_alert_pixbuf());
+	label = gtk_label_new(_("Alerts"));
+	gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 3);
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 3);
+	b = gtk_button_new();
+	if (button)
+		*button = b;
+	if (cb_func)
+		g_signal_connect(G_OBJECT(b), "clicked", G_CALLBACK(cb_func), data);
+	gtk_widget_show_all(hbox);
+	gtk_container_add(GTK_CONTAINER(b), hbox);
+	if (box)
+		{
+		if (pack_start)
+			gtk_box_pack_start(GTK_BOX(box), b, expand, fill, pad);
+		else
+			gtk_box_pack_end(GTK_BOX(box), b, expand, fill, pad);
+		}
+	}
+
 void
 gkrellm_gtk_spin_button(GtkWidget *box, GtkWidget **spin_button, gfloat value,
 		gfloat low, gfloat high, gfloat step0, gfloat step1,
@@ -680,7 +710,8 @@ static GtkWidget
 			*above_button,
 			*below_button;
 
-GtkWidget	*track_gtk_button;
+GtkWidget	*track_gtk_button,
+			*allow_multiple_button;
 GtkWidget	*on_top_button;
 
 
@@ -721,6 +752,9 @@ cb_general(void)
 
 	if (track_gtk_button)
 		_GK.track_gtk_rcfiles = GTK_TOGGLE_BUTTON(track_gtk_button)->active;
+	if (allow_multiple_button)
+		_GK.allow_multiple_instances =
+					GTK_TOGGLE_BUTTON(allow_multiple_button)->active;
 	if (on_top_button)
 		_GK.on_top = GTK_TOGGLE_BUTTON(on_top_button)->active;
 
@@ -914,9 +948,13 @@ create_general_tab(GtkWidget *tab_vbox)
 
 #if !defined(WIN32)
 	gkrellm_gtk_check_button_connected(vbox, &track_gtk_button,
-			_GK.track_gtk_rcfiles, FALSE, FALSE, 10,
+			_GK.track_gtk_rcfiles, FALSE, FALSE, 0,
 			cb_general, NULL,
 			_("Track Gtk theme changes for similarly named themes"));
+	gkrellm_gtk_check_button_connected(vbox, &allow_multiple_button,
+			_GK.allow_multiple_instances, FALSE, FALSE, 0,
+			cb_general, NULL,
+			_("Allow multiple instances"));
 #endif
 
 #if defined(WIN32)
@@ -1970,7 +2008,10 @@ cb_tree_selection_changed(GtkTreeSelection *selection, gpointer data)
 				MONITOR_COLUMN, &selected_monitor,
 				PAGE_COLUMN, &page,
 				-1);
-	if (selected_monitor)
+	if (_GK.debug_level & DEBUG_GUI)
+		printf("tree_selection %p: %s\n", selected_monitor,
+				selected_monitor ? selected_monitor->name : "--");
+	if (selected_monitor && selected_monitor->privat)
 		{
 		page = selected_monitor->privat->config_page;
 		real_create_config(selected_monitor);
