@@ -110,9 +110,10 @@ draw_hostname(void)
 void
 update_host(void)
 	{
-	gint	connect_state;
+	gint		connect_state;
+	static gint	reconnect_timeout;
 
-	if (!_GK.client_mode || !GK.two_second_tick)
+	if (!_GK.client_mode || !GK.second_tick)
 		return;
 
 	/* If we loose the server connection, trigger a hardwired alarm.
@@ -121,9 +122,26 @@ update_host(void)
 	if (connect_state == 2)		/* thread is trying a reconnect */
 		gkrellm_check_alert(server_alert, 1.0);		/* Warning	*/
 	else if (connect_state == 0)	/* Lost connection			*/
+		{
 		gkrellm_check_alert(server_alert, 2.0);		/* Alarm 	*/
+		if (   _GK.client_server_reconnect_timeout > 0
+			&& ++reconnect_timeout > _GK.client_server_reconnect_timeout
+		   )
+			{
+			gkrellm_client_mode_connect_thread();
+			reconnect_timeout = 0;
+			}
+		}
 	else
+		{
 		gkrellm_check_alert(server_alert, 0.0);		/* Alert off	*/
+		reconnect_timeout = 0;
+		if (   _GK.client_server_io_timeout > 0
+			&& _GK.time_now >
+					_GK.client_server_read_time + _GK.client_server_io_timeout
+		   )
+			gkrellm_client_mode_disconnect();
+		}
 	}
 
 static void
