@@ -93,7 +93,8 @@ static gchar		*inet_data_dir;
 static gchar		*text_format,
 					*text_format_locale;
 
-static gint			n_inet_monitors;
+static gint			n_inet_monitors,
+					update_interval  = 1;
 static gint			style_id;
 
 
@@ -233,14 +234,18 @@ gkrellm_inet_client_divert(void (*read_tcp_func)())
             			in->port0_0, in->port0_1, in->port1_0, in->port1_1);
 
 static void
-format_chart_text(InetMon *in, gchar *buf, gint size)
+format_chart_text(InetMon *in, gchar *src_string, gchar *buf, gint size)
 	{
 	GkrellmChart	*cp;
 	gchar			c, *s, *s1;
 	gint			len, value;
 
+	if (!buf || size < 1)
+		return;
 	--size;
 	*buf = '\0';
+	if (!src_string)
+		return;
 	cp = in->chart;
 	for (s = text_format_locale; *s != '\0' && size > 0; ++s)
 		{
@@ -280,7 +285,7 @@ draw_inet_extra(InetMon *in)
 
 	if (!in->extra_info)
 		return;
-	format_chart_text(in, buf, sizeof(buf));
+	format_chart_text(in, text_format_locale, buf, sizeof(buf));
 	gkrellm_draw_chart_text(in->chart, style_id, buf);
 	}
 
@@ -456,11 +461,12 @@ update_inet(void)
 	GList			*list;
 	gchar			buf[32];
 	gint			i;
+	static gint		check_tcp;
 
-	if (inet_mon_list == NULL)
+	if (!inet_mon_list)
 		return;
 
-	if (GK.second_tick || GK.minute_tick || GK.hour_tick)
+	if (GK.second_tick && check_tcp == 0)
 		{
 		for (list = inet_mon_list; list; list = list->next)
 			{
@@ -493,6 +499,8 @@ update_inet(void)
 				list = list->next;
 			}
 		}
+	if (GK.second_tick)
+		check_tcp = (check_tcp + 1) % update_interval;
 
 	for (list = inet_mon_list; list; list = list->next)
 		{
@@ -1015,6 +1023,9 @@ save_inet_config(FILE *f)
 						in->name, in->launch.tooltip_comment);
 		}
 	fprintf(f, "%s text_format all %s\n", INET_CONFIG_KEYWORD, text_format);
+	if (!_GK.client_mode)
+		fprintf(f, "%s update_interval %d\n",
+					INET_CONFIG_KEYWORD, update_interval);
 	}
 
 static gint
@@ -1068,6 +1079,12 @@ load_inet_config(gchar *arg)
 		gkrellm_locale_dup_string(&text_format, item, &text_format_locale);
 		return;
 		}
+	else if (!strcmp(config, "update_interval"))
+		{
+		sscanf(item, "%d", &update_interval);
+		if (update_interval < 1)
+			update_interval = 1;
+		}
 	else if (!strcmp(config, "monitor"))
 		{
 		in = g_new0(InetMon, 1);
@@ -1691,6 +1708,11 @@ cb_text_format(GtkWidget *widget, gpointer data)
 		draw_inet_chart((InetMon *) list->data);
 	}
 
+static void
+cb_update_interval(GtkWidget *entry, GtkSpinButton *spin)
+    {
+    update_interval = gtk_spin_button_get_value_as_int(spin);
+    }
 
 static gchar	*inet_info_text[] =
 {
@@ -1913,6 +1935,12 @@ create_inet_tab(GtkWidget *tab_vbox)
 /* --Setup tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Setup"));
 
+	if (!_GK.client_mode)
+		gkrellm_gtk_spin_button(vbox, NULL,
+					(gfloat) update_interval, 1, 20, 1, 1, 0, 55,
+            		cb_update_interval, NULL, FALSE,
+            		_("Seconds between updates"));
+
 	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Format String for Chart Labels"),
 				4, FALSE, 0, 2);
 	hbox = gtk_hbox_new(FALSE, 0);
