@@ -68,7 +68,7 @@ struct
 
 static GtkWidget *top_window;
 GList			*gkrellm_monitor_list;
-time_t			time_now;
+time_t			gkrellm_time_now;
 
 static GtkItemFactory	*item_factory;
 
@@ -293,14 +293,14 @@ update_monitors()
 	struct tm		*pCur, *pPrev;
 	static time_t	time_prev;
 
-	time(&time_now);
-	GK.second_tick = (time_now == time_prev) ? FALSE : TRUE;
-	time_prev = time_now;
+	time(&_GK.time_now);
+	GK.second_tick = (_GK.time_now == time_prev) ? FALSE : TRUE;
+	time_prev = _GK.time_now;
 
 	if (GK.second_tick)
 		{
 		pPrev = &gkrellm_current_tm;
-		pCur = localtime(&time_now);
+		pCur = localtime(&_GK.time_now);
 		GK.two_second_tick  = ((pCur->tm_sec % 2) == 0) ? TRUE : FALSE;
 		GK.five_second_tick = ((pCur->tm_sec % 5) == 0) ? TRUE : FALSE;
 		GK.ten_second_tick  = ((pCur->tm_sec % 10) == 0) ? TRUE : FALSE;
@@ -562,7 +562,7 @@ top_frame_button_press(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 	|  confused, so put some duct tape here ...
 	*/
 	time(&time_check);
-	if (time_check > time_now + 2 || time_check < time_now)
+	if (time_check > _GK.time_now + 2 || time_check < _GK.time_now)
 		start_timer(_GK.update_HZ);
 
 	if (ev->button == 3)
