@@ -72,6 +72,8 @@ typedef struct
 	gint		enabled;
 	GkrellmLauncher	launch;
 
+	GkrellmAlert *alert;
+
 	guint64		total,		/* Total memory or swap in system */
 				used,		/* Amount of memory (calulated) or swap used  */
 				free,		/* Not used by swap	monitor */
@@ -188,20 +190,24 @@ force_meminfo_update(void)
 	}
 
 static gint
-format_meminfo_data(MeminfoMeter *mm, gchar *buf, gint size)
+format_meminfo_data(MeminfoMeter *mm, gchar *src_string, gchar *buf, gint size)
 	{
 	gulong	t, u, f;
 	gchar	*s, *label;
 	gint	len;
 
+	if (!buf || size < 1)
+		return -1;
 	--size;
 	*buf = '\0';
+	if (!src_string)
+		return -1;
 	label = mm->label;
 	t = MEG(mm->total);
 	u = MEG(mm->used);
 	f = t - u;
 
-	for (s = mm->data_format_shadow; *s != '\0' && size > 0; ++s)
+	for (s = src_string; *s != '\0' && size > 0; ++s)
 		{
 		len = 1;
 		if (*s == '$' && *(s + 1) != '\0')
@@ -240,6 +246,10 @@ format_meminfo_data(MeminfoMeter *mm, gchar *buf, gint size)
 					if (mm == &mem)
 						len = snprintf(buf, size, "%ldM", MEG(mem.cached));
 					break;
+				case 'H':
+						len = snprintf(buf, size, "%s",
+									gkrellm_sys_get_host_name());
+					break;
 				default:
 					*buf = *s;
 					if (size > 1)
@@ -280,7 +290,8 @@ draw_decal_label(MeminfoMeter *mm, gint draw_to_screen)
 		ts_save = d->text_style;
 		d->text_style = *gkrellm_meter_alt_textstyle(mm->style_id);
 
-		changed = format_meminfo_data(mm, buf, sizeof(buf));
+		changed = format_meminfo_data(mm, mm->data_format_shadow,
+					buf, sizeof(buf));
 		w = gkrellm_gdk_string_width(d->text_style.font, buf);
 
 		if (w  > d->w)
@@ -303,6 +314,12 @@ draw_decal_label(MeminfoMeter *mm, gint draw_to_screen)
 	return w;
 	}
 
+static void
+cb_command_process(GkrellmAlert *alert, gchar *src, gchar *dst, gint len,
+			MeminfoMeter *m)
+	{
+	format_meminfo_data(m, src, dst, len);
+	}
 
 static void
 record_activity(gint *pipe, gint modified)
@@ -421,8 +438,8 @@ static void
 update_meminfo(void)
 	{
 	GkrellmChart	*cp;
-	gulong	u, b, c;
-	gint	w_scroll, w;
+	gulong			u, b, c, used;
+	gint			w_scroll, w, full_scale;
 
 	if (! (mem.enabled || swap.enabled || swap_chart.enabled))
 		return;
@@ -442,10 +459,12 @@ update_meminfo(void)
 		}
 	if (mem.enabled)
 		{
-		mem.krell_used->full_scale = (gint) (mem.total >> 12);
-		mem.krell_buffers->full_scale = mem.krell_used->full_scale;
-		mem.krell_cache->full_scale = mem.krell_used->full_scale;
-		u = (gulong) (mem.used >> 12);
+		full_scale = (gint) (mem.total >> 12);
+		gkrellm_set_krell_full_scale(mem.krell_used, full_scale, 1);
+		gkrellm_set_krell_full_scale(mem.krell_buffers, full_scale, 1);
+		gkrellm_set_krell_full_scale(mem.krell_cache, full_scale, 1);
+
+		used = u = (gulong) (mem.used >> 12);
 		b = u + (gulong)(mem.buffers >> 12);
 		c = b + (gulong)(mem.cached >> 12);
 		if (   (mem.label_is_data && mon_in_motion && x_moved)
@@ -456,18 +475,24 @@ update_meminfo(void)
 		gkrellm_update_krell(mem.panel, mem.krell_buffers, b);
 		gkrellm_update_krell(mem.panel, mem.krell_cache, c);
 		record_activity(mem_pipe, mem.krell_used->modified);
+		if (mem.alert && GK.second_tick)
+			gkrellm_check_alert(mem.alert,
+						100.0 * (gfloat) used / (gfloat) full_scale);
 		}
 	if (swap.enabled)
 		{
-		gkrellm_set_krell_full_scale(swap.krell_used,
-					(gint) (swap.total >> 12), 1);
-		u = (gulong)(swap.used >> 12);
+		full_scale = (gint) (swap.total >> 12);
+		gkrellm_set_krell_full_scale(swap.krell_used, full_scale, 1);
+		used = u = (gulong)(swap.used >> 12);
 		if (   (swap.label_is_data && mon_in_motion && x_moved)
 			|| swap.mouse_entered
 		   )
 			u = 0;
 		gkrellm_update_krell(swap.panel, swap.krell_used, u);
 		record_activity(swap_pipe, swap.krell_used->modified);
+		if (swap.alert && GK.second_tick)
+			gkrellm_check_alert(swap.alert,
+						100.0 * (gfloat) used / (gfloat) full_scale);
 		}
 	if (swap.krell_delta && swap.enabled)
 		gkrellm_update_krell(swap.panel, swap.krell_delta,
@@ -599,7 +624,8 @@ cb_panel_motion(GtkWidget *widget, GdkEventButton *ev)
 	d = mon_in_motion->decal_label;
 	font = gkrellm_meter_alt_textstyle(mon_in_motion->style_id)->font;
 
-	format_meminfo_data(mon_in_motion, buf, sizeof(buf));
+	format_meminfo_data(mon_in_motion, mon_in_motion->data_format_shadow,
+				buf, sizeof(buf));
 	w = gkrellm_gdk_string_width(font, buf);
 	if (w > d->w)
 		{
@@ -915,6 +941,31 @@ create_mem(GtkWidget *vbox, gint first_create)
 
 #define	MEM_CONFIG_KEYWORD	"meminfo"
 
+static void
+cb_alert_trigger(GkrellmAlert *alert, MeminfoMeter *m)
+	{
+	/* Full panel alert, default decal.
+	*/
+	alert->panel = m->panel;
+	}
+
+static void
+create_alert(MeminfoMeter *m)
+	{
+	gchar	*label;
+
+	if (m == &mem)
+		label = _("Memory");
+	else
+		label = _("Swap");
+	m->alert = gkrellm_alert_create(NULL, label,
+			_("Percent Usage"),
+			TRUE, FALSE, TRUE,
+			100, 10, 1, 10, 0);
+	gkrellm_alert_trigger_connect(m->alert, cb_alert_trigger, m);
+	gkrellm_alert_command_process_connect(m->alert, cb_command_process, m);
+	}
+
 static void
 save_meminfo_config(FILE *f)
 	{
@@ -941,13 +992,19 @@ save_meminfo_config(FILE *f)
 				swap.data_format);
 
 	fprintf(f, "%s text_format %s\n", MEM_CONFIG_KEYWORD, text_format);
+
+	if (mem.alert)
+		gkrellm_save_alertconfig(f, mem.alert, MEM_CONFIG_KEYWORD, "mem");
+	if (swap.alert)
+		gkrellm_save_alertconfig(f, swap.alert, MEM_CONFIG_KEYWORD, "swap");
 	}
 
 static void
 load_meminfo_config(gchar *arg)
 	{
-	gchar	config[32], item[CFG_BUFSIZE];
-	gint	n;
+	MeminfoMeter *m = NULL;
+	gchar		config[32], name[16], item[CFG_BUFSIZE], item1[CFG_BUFSIZE];
+	gint		n;
 
 	n = sscanf(arg, "%31s %[^\n]", config, item);
 	if (n == 2)
@@ -985,6 +1042,22 @@ load_meminfo_config(gchar *arg)
 						&swap.data_format_shadow);
 		else if (!strcmp(config, "text_format"))
 			gkrellm_locale_dup_string(&text_format, item, &text_format_locale);
+		else if (!strcmp(config, GKRELLM_ALERTCONFIG_KEYWORD))
+			{
+			if (sscanf(item, "%16s %[^\n]", name, item1) == 2)
+				{
+				if (!strcmp(name, "mem"))
+					m = &mem;
+				else if (!strcmp(name, "swap"))
+					m = & swap;
+				if (m)
+					{
+					if (!m->alert)
+						create_alert(m);
+					gkrellm_load_alertconfig(&m->alert, item1);
+					}
+				}
+			}
 		}
 	}
 
@@ -1000,6 +1073,9 @@ static GtkWidget	*mem_format_combo,
 
 static GtkWidget	*text_format_combo;
 
+static GtkWidget	*mem_alert_button,
+					*swap_alert_button;
+
 
 static void
 cb_text_format(GtkWidget *widget, gpointer data)
@@ -1019,6 +1095,7 @@ cb_mem_enable(GtkWidget *button, gpointer data)
 	enabled = GTK_TOGGLE_BUTTON(button)->active;
 	gkrellm_panel_enable_visibility(mem.panel, enabled, &mem.enabled);
 	spacer_visibility();
+	gtk_widget_set_sensitive(mem_alert_button, enabled);
 	}
 
 static void
@@ -1029,6 +1106,7 @@ cb_swap_enable(GtkWidget *button, gpointer data)
 	enabled = GTK_TOGGLE_BUTTON(button)->active;
 	gkrellm_panel_enable_visibility(swap.panel, enabled, &swap.enabled);
 	spacer_visibility();
+	gtk_widget_set_sensitive(swap_alert_button, enabled);
 	}
 
 static void
@@ -1097,6 +1175,13 @@ cb_swap_format(GtkWidget *widget, gpointer data)
 		swap.decal_label->value = -1;
 	}
 
+static void
+cb_set_alert(GtkWidget *button, MeminfoMeter *m)
+	{
+	if (!m->alert)
+		create_alert(m);
+	gkrellm_alert_config_window(&m->alert);
+	}
 
 #define	DEFAULT_TEXT_FORMAT	"$T"
 
@@ -1132,6 +1217,9 @@ N_("For memory only:\n"),
 N_("\t$s    shared MiB\n"),
 N_("\t$b    buffered MiB\n"),
 N_("\t$c    cached MiB\n"),
+"\n",
+N_("Substitution variables may be used in alert commands.\n"),
+
 "\n",
 N_("<h>Mouse Button Actions:\n"),
 N_("<b>\tLeft "),
@@ -1164,16 +1252,28 @@ create_meminfo_tab(GtkWidget *tab_vbox)
 				swap_chart.enabled, FALSE, FALSE, 0,
 				cb_swap_chart_enable, NULL,
 				_("Enable swap pages in/out chart"));
-    gkrellm_gtk_check_button_connected(vbox1, NULL,
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 5);
+    gkrellm_gtk_check_button_connected(hbox, NULL,
 				swap.enabled, FALSE, FALSE, 0,
 				cb_swap_enable, GINT_TO_POINTER(1),
 				_("Enable swap meter"));
+	gkrellm_gtk_alert_button(hbox, &swap_alert_button, FALSE, FALSE, 4, FALSE,
+				cb_set_alert, &swap);
+	if (!swap.enabled)
+		gtk_widget_set_sensitive(swap_alert_button, FALSE);
 
 	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Memory"), 4, FALSE, 0, 2);
-    gkrellm_gtk_check_button_connected(vbox1, NULL,
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 5);
+    gkrellm_gtk_check_button_connected(hbox, NULL,
 				mem.enabled, FALSE, FALSE, 0,
 				cb_mem_enable, NULL,
 				_("Enable memory meter"));
+	gkrellm_gtk_alert_button(hbox, &mem_alert_button, FALSE, FALSE, 4, FALSE,
+				cb_set_alert, &mem);
+	if (!mem.enabled)
+		gtk_widget_set_sensitive(mem_alert_button, FALSE);
     gkrellm_gtk_check_button_connected(vbox1, NULL,
 				mem.all_krells, FALSE, FALSE, 0,
 				cb_all_krells, NULL,
