@@ -38,6 +38,7 @@ typedef struct
 					*parent_vbox;
 	GtkWidget		*enable_button,
 					*force_button,
+					*alert_button,
 					*label_entry;
 	GkrellmChart	*chart;
 	GkrellmChartdata *rx_cd,
@@ -61,6 +62,12 @@ typedef struct
 						force_up;
 	gchar				*label;
 
+	GkrellmAlert		*alert;
+	GtkWidget			*alert_config_rx_button,
+						*alert_config_tx_button;
+	gboolean			alert_uses_rx,
+						alert_uses_tx;
+
 	gulong				rx_old,
 						tx_old;
 	gint				rx_current,
@@ -76,7 +83,6 @@ typedef struct
 	GtkWidget			*launch_entry,
 						*tooltip_entry;
 
-
 	gboolean			up,
 						up_prev,
 						up_event,
@@ -148,7 +154,7 @@ new_net(gchar *name)
 	net->label = g_strdup("");
 	net->launch.command = g_strdup("");
 	net->launch.tooltip_comment = g_strdup("");
-
+	net->alert_uses_rx = net->alert_uses_tx = TRUE;
 	net_mon_list = g_list_insert_sorted(net_mon_list, net,
 				(GCompareFunc) strcmp_net_name);
 
@@ -888,18 +894,22 @@ static gchar    *text_format,
 				*text_format_locale;
 
 static void
-format_chart_text(NetMon *net, gchar *buf, gint size)
+format_net_data(NetMon *net, gchar *src_string, gchar *buf, gint size)
 	{
 	GkrellmChart	*cp;
 	gchar	c, *s, *s1;
 	gint	len, bytes;
 	gdouble	fbytes;
 
+	if (!buf || size < 1)
+		return;
 	--size;
 	*buf = '\0';
+	if (!src_string)
+		return;
 	cp = net->chart;
 	net->totals_shown = FALSE;
-	for (s = text_format_locale; *s != '\0' && size > 0; ++s)
+	for (s = src_string; *s != '\0' && size > 0; ++s)
 		{
 		len = 1;
 		if (*s == '$' && *(s + 1) != '\0')
@@ -929,6 +939,10 @@ format_chart_text(NetMon *net, gchar *buf, gint size)
 					s1 = net->label;
 				len = snprintf(buf, size, "%s", s1);
 				}
+			else if (c == 'I')
+				len = snprintf(buf, size, "%s", net->name);
+			else if (c == 'H')
+				len = snprintf(buf, size, "%s", gkrellm_sys_get_host_name());
 			else
 				{
 				*buf = *s;
@@ -967,10 +981,17 @@ draw_net_chart_labels(NetMon *net)
 
 	if (!net->chart_labels)
 		return;
-	format_chart_text(net, buf, sizeof(buf));
+	format_net_data(net, text_format_locale, buf, sizeof(buf));
 	gkrellm_draw_chart_text(cp, net_style_id, buf);
 	}
 
+static void
+cb_command_process(GkrellmAlert *alert, gchar *src, gchar *dst, gint len,
+			NetMon *net)
+	{
+	format_net_data(net, src, dst, len);
+	}
+
 static gboolean
 draw_totals_buttons(NetMon *net)
 	{
@@ -1309,8 +1330,10 @@ create_net_timer(GtkWidget *vbox, gint first_create)
 static void
 update_net(void)
 	{
-	GList		*list;
-	NetMon		*net;
+	GList			*list;
+	NetMon			*net;
+	GkrellmPanel	*p;
+	gint			bytes;
 
 	/* If sysdep code is not reporting route up/down events, then
 	|  gkrellm_net_assign_data() sets a net as up if data is assigned for it.
@@ -1351,6 +1374,7 @@ update_net(void)
 		net = (NetMon *) list->data;
 		if (!net->chart)
 			continue;
+		p = net->chart->panel;
 		if (net->up || net->force_up)
 			{
 			if (net->rx > net->rx_old)
@@ -1373,11 +1397,21 @@ update_net(void)
 			net->tx_totalA += (gdouble) net->tx_current;
 			net->rx_totalB += (gdouble) net->rx_current;
 			net->tx_totalB += (gdouble) net->tx_current;
+			if (net->alert)
+				{
+				bytes = 0;
+				if (net->alert_uses_rx)
+					bytes += net->rx_current;
+				if (net->alert_uses_tx)
+					bytes += net->tx_current;
+				gkrellm_check_alert(net->alert, bytes);
+				}
+			gkrellm_panel_label_on_top_of_decals(p,
+						gkrellm_alert_decal_visible(net->alert));
 			refresh_net_chart(net);
 			}
-		gkrellm_update_krell(net->chart->panel, KRELL(net->chart->panel),
-					net->tx + net->rx);
-		gkrellm_draw_panel_layers(net->chart->panel);
+		gkrellm_update_krell(p, KRELL(p), net->tx + net->rx);
+		gkrellm_draw_panel_layers(p);
 		}
 	if (GK.second_tick)
 		{
@@ -1606,6 +1640,35 @@ create_timer(GtkWidget *vbox, gint first_create)
 
 #define	NET_CONFIG_KEYWORD	"net"
 
+static void
+cb_alert_trigger(GkrellmAlert *alert, NetMon *net)
+	{
+	/* Full panel alert, default decal.
+	*/
+	alert->panel = net->chart->panel;
+	}
+
+static void
+create_alert(NetMon *net)
+	{
+	static void cb_alert_config(GkrellmAlert *ap, NetMon *net);
+	static void	cb_alert_config_create(GkrellmAlert *ap, GtkWidget *vbox,
+					NetMon *net);
+
+	net->alert = gkrellm_alert_create(NULL, net->name,
+				_("Bytes per second"),
+				TRUE, FALSE, TRUE,
+				1e10, 1000, 1000, 10000, 0);
+	gkrellm_alert_delay_config(net->alert, 1, 60 * 60, 0);
+
+	gkrellm_alert_trigger_connect(net->alert, cb_alert_trigger, net);
+	gkrellm_alert_config_connect(net->alert, cb_alert_config, net);
+	gkrellm_alert_config_create_connect(net->alert,
+							cb_alert_config_create, net);
+	gkrellm_alert_command_process_connect(net->alert, cb_command_process, net);
+	}
+
+
 static void
 save_net_config(FILE *f)
 	{
@@ -1628,6 +1691,14 @@ save_net_config(FILE *f)
 		if (*(net->launch.tooltip_comment))
 			fprintf(f, "%s tooltip %s %s\n", NET_CONFIG_KEYWORD,
 					net->name, net->launch.tooltip_comment);
+		if (net->alert)
+			{
+			gkrellm_save_alertconfig(f, net->alert,
+						NET_CONFIG_KEYWORD, net->name);
+			fprintf(f, "%s extra_alert_config %s %d %d\n", NET_CONFIG_KEYWORD,
+						net->name,
+						net->alert_uses_rx, net->alert_uses_tx);
+			}
 		}
 	fprintf(f, "%s timer_enabled %d\n", NET_CONFIG_KEYWORD,
 						timer_button_enabled);
@@ -1694,6 +1765,14 @@ load_net_config(gchar *arg)
 			return;
 		if (!strcmp(config, GKRELLM_CHARTCONFIG_KEYWORD))
 			gkrellm_load_chartconfig(&net->chart_config, item1, 2);
+		else if (!strcmp(config, GKRELLM_ALERTCONFIG_KEYWORD))
+			{
+			if (!net->alert)
+				create_alert(net);
+			gkrellm_load_alertconfig(&net->alert, item1);
+			}
+		else if (!strcmp(config, "extra_alert_config"))
+			sscanf(item1, "%d %d", &net->alert_uses_rx, &net->alert_uses_tx);
 		else if (!strcmp(config, "label"))
 			gkrellm_dup_string(&net->label, item1);
 		else if (!strcmp(config, "launch"))
@@ -1760,6 +1839,7 @@ cb_enable(GtkWidget *button, NetMon *net)
 			net->force_up = FALSE;
 			}
 		}
+	gtk_widget_set_sensitive(net->alert_button, net->enabled);
 	if (net == net_timed && !net->enabled && timer_iface_combo)
 		gtk_entry_set_text(
 					GTK_ENTRY(GTK_COMBO(timer_iface_combo)->entry), "none");
@@ -1797,6 +1877,103 @@ cb_label_entry(GtkWidget *widget, NetMon *net)
 	refresh_net_chart(net);
 	}
 
+static void
+cb_set_alert(GtkWidget *button, NetMon *net)
+	{
+	
+#if 0
+	GtkTreeModel	*model;
+	GtkTreePath		*path;
+	GtkTreeIter		iter;
+	NetMon			*net;
+
+	if (!row_reference)
+		return;
+	model = gtk_tree_view_get_model(treeview);
+	path = gtk_tree_row_reference_get_path(row_reference);
+	gtk_tree_model_get_iter(model, &iter, path);
+	gtk_tree_model_get(model, &iter, NETMON_COLUMN, &net, -1);
+#endif
+
+	if (!net->alert)
+		create_alert(net);
+	gkrellm_alert_config_window(&net->alert);
+
+#if 0
+	gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
+				ALERT_COLUMN, net->alert, -1);
+#endif
+	}
+
+  /* Callback for a created or destroyed alert.  Find the sensor in the model
+  |  and set the IMAGE_COLUMN.
+  */
+static void
+cb_alert_config(GkrellmAlert *ap, NetMon *net)
+    {
+#if 0
+	GtkTreeModel	*model;
+	GtkTreeIter		iter;
+	NetMon			*net_test;
+	GdkPixbuf		*pixbuf;
+	gchar			node[2];
+	gint			i;
+
+	if (!gkrellm_config_window_shown())
+		return;
+	model = gtk_tree_view_get_model(treeview);
+	pixbuf = ap->activated ? gkrellm_alert_pixbuf() : NULL;
+	for (i = 0; i < 2; ++i)
+		{
+		node[0] = '0' + i;      /* toplevel Primary or Secondary node */
+		node[1] = '\0';
+		if (get_child_iter(model, node, &iter))
+			do
+				{
+				gtk_tree_model_get(model, &iter, NETMON_COLUMN, &net_test, -1);
+				if (net != net_test)
+					continue;
+				gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
+							IMAGE_COLUMN, pixbuf, -1);
+				return;
+				}
+			while (gtk_tree_model_iter_next(model, &iter));
+		}
+#endif
+	net->alert_uses_rx =
+			GTK_TOGGLE_BUTTON(net->alert_config_rx_button)->active;
+	net->alert_uses_tx =
+			GTK_TOGGLE_BUTTON(net->alert_config_tx_button)->active;
+	}
+
+static void
+cb_alert_config_button(GtkWidget *button, NetMon *net)
+	{
+	gboolean	read, write;
+
+	read = GTK_TOGGLE_BUTTON(net->alert_config_rx_button)->active;
+	write = GTK_TOGGLE_BUTTON(net->alert_config_tx_button)->active;
+	if (!read && !write)
+		{
+		gtk_toggle_button_set_active(
+				GTK_TOGGLE_BUTTON(net->alert_config_rx_button), TRUE);
+		gtk_toggle_button_set_active(
+				GTK_TOGGLE_BUTTON(net->alert_config_tx_button), TRUE);
+		}
+	}
+
+static void
+cb_alert_config_create(GkrellmAlert *ap, GtkWidget *vbox, NetMon *net)
+    {
+    gkrellm_gtk_check_button_connected(vbox, &net->alert_config_rx_button,
+                net->alert_uses_rx, FALSE, FALSE, 2,
+                cb_alert_config_button, net, _("rx bytes"));
+    gkrellm_gtk_check_button_connected(vbox, &net->alert_config_tx_button,
+                net->alert_uses_tx, FALSE, FALSE, 2,
+                cb_alert_config_button, net, _("tx bytes"));
+    }
+
+
 static void
 cb_timer_enable(GtkWidget *button, gpointer data)
 	{
@@ -1921,9 +2098,11 @@ N_("\t$t    transmit bytes\n"),
 N_("\t$O    cumulative receive + transmit bytes\n"),
 N_("\t$i    cumulative receive bytes\n"),
 N_("\t$o    cumulative transmit bytes\n"),
-N_("\t$L    the optional chart label\n")
-}
-;
+N_("\t$L    the optional chart label\n"),
+N_("\t$I    the net interface name\n"),
+"\n",
+N_("Substitution variables may be used in alert commands.\n")
+};
 
 static void
 create_net_tab(GtkWidget *tab_vbox)
@@ -2016,10 +2195,17 @@ create_net_tab(GtkWidget *tab_vbox)
 		vbox = gkrellm_gtk_framed_notebook_page(tabs, net->name);
 
 		snprintf(buf, sizeof(buf), _("Enable %s"), net->name);
-		gkrellm_gtk_check_button_connected(vbox,
+		hbox = gtk_hbox_new (FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 4);
+		gkrellm_gtk_check_button_connected(hbox,
 						&net->enable_button, net->enabled,
-						FALSE, FALSE, 8,
+						FALSE, FALSE, 0,
 						cb_enable, net, buf);
+		gkrellm_gtk_alert_button(hbox, &net->alert_button, FALSE, FALSE, 4,
+					FALSE, cb_set_alert, net);
+		gtk_widget_set_sensitive(net->alert_button, net->enabled);
+
+
 		if (net_config_use_routed)
 			{
 			gkrellm_gtk_check_button_connected(vbox, &net->force_button,
