@@ -55,6 +55,9 @@ ProcMon	proc;
 void	(*read_proc_data)();
 void	(*read_user_data)();
 
+GkrellmAlert	*load_alert,
+				*processes_alert,
+				*users_alert;
 
 static gboolean
 setup_proc_interface(void)
@@ -111,16 +114,20 @@ static gchar	*text_format,
 				*text_format_locale;
 
 static void
-format_chart_text(ProcMon *p, gchar *buf, gint size)
+format_proc_data(ProcMon *p, gchar *src_string, gchar *buf, gint size)
 	{
 	GkrellmChart	*cp;
 	gchar			c, *s;
 	gint			len, value;
 
+	if (!buf || size < 1)
+		return;
 	--size;
 	*buf = '\0';
+	if (!src_string)
+		return;
 	cp = p->chart;
-	for (s = text_format_locale; *s != '\0' && size > 0; ++s)
+	for (s = src_string; *s != '\0' && size > 0; ++s)
 		{
 		len = 1;
 		if (*s == '$' && *(s + 1) != '\0')
@@ -132,6 +139,8 @@ format_chart_text(ProcMon *p, gchar *buf, gint size)
 			else if (c == 'F' && fork_scaling > 0)
 				len = snprintf(buf, size, "%d",
 						gkrellm_get_chart_scalemax(cp) / fork_scaling);
+			else if (c == 'H')
+				len = snprintf(buf, size, "%s", gkrellm_sys_get_host_name());
 			else if (c == 'l')
 				len = snprintf(buf, size, "%.1f", p->fload);
 			else if (c == 'p')
@@ -160,10 +169,17 @@ draw_proc_extra(void)
 
 	if (!proc.chart || !proc.extra_info)
 		return;
-	format_chart_text(&proc, buf, sizeof(buf));
+	format_proc_data(&proc, text_format_locale, buf, sizeof(buf));
 	gkrellm_draw_chart_text(proc.chart, style_id, buf);
 	}
 
+static void
+cb_command_process(GkrellmAlert *alert, gchar *src, gchar *dst, gint len,
+			gpointer data)
+	{
+	format_proc_data(&proc, src, dst, len);
+	}
+
 static void
 refresh_proc_chart(GkrellmChart *cp)
 	{
@@ -193,7 +209,7 @@ draw_sensor_decals(void)
 		}
 	else
 		{
-		toggle = time_now & 2;
+		toggle = _GK.time_now & 2;
 		if (show_fan && (toggle || !show_temperature))
 			{
 			gkrellm_sensor_read_fan(proc.sensor_fan, &f);
@@ -229,9 +245,11 @@ update_proc(void)
 	if (!proc.enabled)
 		return;
 	(*read_proc_data)();
-	if (GK.five_second_tick && proc.extra_info)
+	if (GK.five_second_tick)
+		{
 		(*read_user_data)();
-
+		gkrellm_check_alert(users_alert, proc.n_users);
+		}
 	cp = proc.chart;
 	p = cp->panel;
 	gkrellm_update_krell(p, KRELL(p), proc.n_forks);
@@ -245,6 +263,13 @@ update_proc(void)
 		load = (int) (LOAD_SCALING * proc.fload);
 		gkrellm_store_chartdata(cp, 0, load, fork_scaling * proc.n_forks);
 		refresh_proc_chart(cp);
+		gkrellm_check_alert(load_alert, proc.fload);
+		gkrellm_check_alert(processes_alert, proc.n_processes);
+		gkrellm_panel_label_on_top_of_decals(p,
+				gkrellm_alert_decal_visible(load_alert) ||
+				gkrellm_alert_decal_visible(users_alert) ||
+				gkrellm_alert_decal_visible(processes_alert));
+
 		}
 	if (   (GK.two_second_tick && !sensor_separate_mode)
 		|| (GK.five_second_tick && sensor_separate_mode)
@@ -574,6 +599,77 @@ create_proc(GtkWidget *vbox, gint first_create)
 
 #define	PROC_CONFIG_KEYWORD	"proc"
 
+static void
+cb_alert_trigger(GkrellmAlert *alert, gpointer data)
+	{
+	GkrellmPanel		*p;
+	GkrellmAlertdecal   *ad;
+	GkrellmDecal        *ds, *df;
+	gint				x, w;
+	
+	p = proc.chart->panel;
+	alert->panel = p;
+	ds = proc.sensor_decal;
+	df = proc.fan_decal;
+	ad = &alert->ad;
+	if (gkrellm_is_decal_visible(ds) && !gkrellm_is_decal_visible(df))
+		w = ds->x - 1;
+	else
+		w = p->w;
+	w /= 3;
+	if (w < 2)
+		w = 2;
+	if (alert == load_alert)
+		x = 0;
+	else if (alert == users_alert)
+		x = w;
+	else
+		x = 2 * w;
+	ad->x = x;
+	ad->y = 0;
+	ad->w = w;
+	ad->h = p->h;
+	gkrellm_render_default_alert_decal(alert);
+	}
+
+static void
+create_load_alert(void)
+	{
+	load_alert = gkrellm_alert_create(NULL, _("Load"),
+			NULL,
+			TRUE, FALSE, TRUE,
+			20, 1, 0.5, 1, 2);
+	gkrellm_alert_delay_config(load_alert, 1, 10000, 0);
+
+	gkrellm_alert_trigger_connect(load_alert, cb_alert_trigger, NULL);
+	gkrellm_alert_command_process_connect(load_alert,
+			cb_command_process, NULL);
+	}
+
+static void
+create_users_alert(void)
+	{
+	users_alert = gkrellm_alert_create(NULL, _("Users"),
+			NULL,
+			TRUE, FALSE, TRUE,
+			100000, 2, 1, 10, 0);
+	gkrellm_alert_trigger_connect(users_alert, cb_alert_trigger, NULL);
+	gkrellm_alert_command_process_connect(users_alert,
+			cb_command_process, NULL);
+	}
+
+static void
+create_processes_alert(void)
+	{
+	processes_alert = gkrellm_alert_create(NULL, _("Processes"),
+			NULL,
+			TRUE, FALSE, TRUE,
+			100000, 10, 1, 10, 0);
+	gkrellm_alert_trigger_connect(processes_alert, cb_alert_trigger, NULL);
+	gkrellm_alert_command_process_connect(processes_alert,
+			cb_command_process, NULL);
+	}
+
 static void
 save_proc_config(FILE *f)
 	{
@@ -587,12 +683,19 @@ save_proc_config(FILE *f)
 	fprintf(f, "%s text_format %s\n", PROC_CONFIG_KEYWORD, text_format);
 	gkrellm_save_chartconfig(f, proc.chart_config,
 				PROC_CONFIG_KEYWORD, NULL);
+	if (load_alert)
+		gkrellm_save_alertconfig(f, load_alert, PROC_CONFIG_KEYWORD, "load");
+	if (users_alert)
+		gkrellm_save_alertconfig(f, users_alert, PROC_CONFIG_KEYWORD, "users");
+	if (processes_alert)
+		gkrellm_save_alertconfig(f, processes_alert,
+					PROC_CONFIG_KEYWORD, "processes");
 	}
 
 static void
 load_proc_config(gchar *arg)
 	{
-	gchar	config[32], item[CFG_BUFSIZE];
+	gchar	config[32], name[16], item[CFG_BUFSIZE], item1[CFG_BUFSIZE];
 	gint	n;
 
 	n = sscanf(arg, "%31s %[^\n]", config, item);
@@ -610,6 +713,30 @@ load_proc_config(gchar *arg)
 			gkrellm_locale_dup_string(&text_format, item, &text_format_locale);
 		else if (!strcmp(config, GKRELLM_CHARTCONFIG_KEYWORD))
 			gkrellm_load_chartconfig(&proc.chart_config, item, 2);
+		else if (!strcmp(config, GKRELLM_ALERTCONFIG_KEYWORD))
+			{
+			if (sscanf(item, "%16s %[^\n]", name, item1) == 2)
+				{
+				if (!strcmp(name, "load"))
+					{
+					if (!load_alert)
+						create_load_alert();
+					gkrellm_load_alertconfig(&load_alert, item1);
+					}
+				else if (!strcmp(name, "users"))
+					{
+					if (!users_alert)
+						create_users_alert();
+					gkrellm_load_alertconfig(&users_alert, item1);
+					}
+				else if (!strcmp(name, "processes"))
+					{
+					if (!processes_alert)
+						create_processes_alert();
+					gkrellm_load_alertconfig(&processes_alert, item1);
+					}
+				}
+			}
 		}
 	}
 
@@ -622,6 +749,10 @@ static GtkWidget	*proc_launch_entry,
 
 static GtkWidget	*text_format_combo;
 
+static GtkWidget	*load_alert_button,
+					*users_alert_button,
+					*processes_alert_button;
+
 
 static gboolean
 fix_panel(void)
@@ -686,6 +817,34 @@ cb_enable(GtkWidget *button, gpointer data)
 		gkrellm_spacers_hide(mon_proc);
 		}
 	proc.enabled = enabled;
+
+	gtk_widget_set_sensitive(load_alert_button, enabled);
+	gtk_widget_set_sensitive(users_alert_button, enabled);
+	gtk_widget_set_sensitive(processes_alert_button, enabled);
+	}
+
+static void
+cb_load_alert(GtkWidget *button, gpointer data)
+	{
+	if (!load_alert)
+		create_load_alert();
+	gkrellm_alert_config_window(&load_alert);
+	}
+
+static void
+cb_users_alert(GtkWidget *button, gpointer data)
+	{
+	if (!users_alert)
+		create_users_alert();
+	gkrellm_alert_config_window(&users_alert);
+	}
+
+static void
+cb_processes_alert(GtkWidget *button, gpointer data)
+	{
+	if (!processes_alert)
+		create_processes_alert();
+	gkrellm_alert_config_window(&processes_alert);
 	}
 
 static void
@@ -716,13 +875,15 @@ N_("\t$F    maximum chart value (forks)\n"),
 N_("\t$l    load\n"),
 N_("\t$f    forks\n"),
 N_("\t$p    processes\n"),
-N_("\t$u    users\n")
+N_("\t$u    users\n"),
+"\n",
+N_("Substitution variables may be used in alert commands.\n")
 };
 
 static void
 create_proc_tab(GtkWidget *tab_vbox)
 	{
-	GtkWidget	*tabs, *table, *vbox, *vbox1, *hbox, *text;
+	GtkWidget	*tabs, *table, *vbox, *vbox1, *hbox, *text, *label;
 	GList		*list;
 	gint		i;
 
@@ -731,20 +892,47 @@ create_proc_tab(GtkWidget *tab_vbox)
 	gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);
 
 /* ---Options tab */
-	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Setup"));
-
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Options"), 4, FALSE, 0, 2);
-	gkrellm_gtk_check_button_connected(vbox1, NULL,
+	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Options"));
+	gkrellm_gtk_check_button_connected(vbox, NULL,
 				proc.enabled, FALSE, FALSE, 4,
 				cb_enable, NULL,
 				_("Enable Proc chart"));
 
 	if (gkrellm_sensors_available())
-		gkrellm_gtk_check_button_connected(vbox1, NULL,
+		gkrellm_gtk_check_button_connected(vbox, NULL,
 					sensor_separate_mode, FALSE, FALSE, 0,
 					cb_sensor_separate, NULL,
 		_("Draw fan and temperature values separately (not alternating)."));
 
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, TRUE, 5);
+	gkrellm_gtk_alert_button(hbox, &processes_alert_button, FALSE, FALSE, 4,
+				TRUE, cb_processes_alert, NULL);
+	label = gtk_label_new(_("Processes"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 4);
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, TRUE, 5);
+	gkrellm_gtk_alert_button(hbox, &users_alert_button, FALSE, FALSE, 4, TRUE,
+				cb_users_alert, NULL);
+	label = gtk_label_new(_("Users"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 4);
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, TRUE, 5);
+	gkrellm_gtk_alert_button(hbox, &load_alert_button, FALSE, FALSE, 4, TRUE,
+				cb_load_alert, NULL);
+	label = gtk_label_new(_("Load"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 4);
+
+	if (!proc.enabled)
+		{
+		gtk_widget_set_sensitive(load_alert_button, FALSE);
+		gtk_widget_set_sensitive(users_alert_button, FALSE);
+		gtk_widget_set_sensitive(processes_alert_button, FALSE);
+		}
+
+	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Setup"));
 	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Format String for Chart Labels"),
 				4, FALSE, 0, 2);
 	hbox = gtk_hbox_new(FALSE, 0);
