@@ -597,7 +597,6 @@ add_sensor_monitor(Sensor *sr, GkrellmPanel **p, GList **smon_list)
 	{
 	SensorMon		*smon;
 	gboolean		set_loc;
-	static void		cb_alert_config(GkrellmAlert *ap, Sensor *sr);
 
 	sr->smon = NULL;
 	if (!sr->enabled)
@@ -1346,6 +1345,7 @@ create_sensor_alert(Sensor *s)
 				FALSE, TRUE, TRUE, 20000, 0, 100, 1000, 0);
 	else
 		return;
+	gkrellm_alert_delay_config(s->alert, 5, 60, 0);
 	gkrellm_alert_trigger_connect(s->alert, s->cb_alert, s->cb_alert_data);
 	gkrellm_alert_config_connect(s->alert, cb_alert_config, s);
 	}
@@ -1535,9 +1535,9 @@ set_tree_store_model_data(GtkTreeStore *tree, GtkTreeIter *iter, Sensor *s)
 			VISIBLE_COLUMN, TRUE,
 			-1);
 	if (s->alert)
-	gtk_tree_store_set(tree, iter,
-			IMAGE_COLUMN, gkrellm_alert_pixbuf(),
-			-1);
+		gtk_tree_store_set(tree, iter,
+				IMAGE_COLUMN, gkrellm_alert_pixbuf(),
+				-1);
 	}
 
 static void
@@ -1649,7 +1649,7 @@ cb_alert_config(GkrellmAlert *ap, Sensor *sr)
 	gchar			node[2];
 	gint			i;
 
-	if (!treeview)
+	if (!gkrellm_config_window_shown())
 		return;
 	model = gtk_tree_view_get_model(treeview);
 	pixbuf = ap->activated ? gkrellm_alert_pixbuf() : NULL;
@@ -1671,7 +1671,7 @@ cb_alert_config(GkrellmAlert *ap, Sensor *sr)
 		}
 	}
 
-  /* Allow desination drops only on depth 2 paths and don't allow drops from
+  /* Allow destination drops only on depth 2 paths and don't allow drops from
   |  source depths of 1 (top level nodes).  Also disallow drags from one sensor
   |  type to another. Note: from some reason if I allow drops on depth 3 nodes
   |  (destination is on top of a second level node) I am not getting
