@@ -81,7 +81,12 @@ gk_sysctlnametomib(const char *name, int *mibp, size_t *lenp)
 /* ===================================================================== */
 /* CPU monitor interface */
 
+#include <osreldate.h>
+#if __FreeBSD_version >= 500101
+#include <sys/resource.h>
+#else
 #include <sys/dkstat.h>
+#endif
 #include <kvm.h>
 
 
@@ -301,8 +306,6 @@ gkrellm_sys_proc_read_users(void)
 /* ===================================================================== */
 /* Disk monitor interface */
 
-#include <osreldate.h>
-#include <sys/dkstat.h>
 #if __FreeBSD_version >= 300000
 #include <devstat.h>
 static struct statinfo	statinfo_cur;
