@@ -832,45 +832,43 @@ gkrellm_sys_net_read_data(void)
 gboolean
 gkrellm_sys_net_isdn_online(void)
 	{
-	FILE	*f = 0;
-	char	buffer[512], *p, *end;
-	int		i;
+	gint	f = 0;
+	gchar	buffer[BUFSIZ], *p, *end;
 
-	if (   (f = fopen ("/dev/isdninfo", "r")) == NULL
-		&& (f = fopen("/dev/isdn/isdninfo", "r")) == NULL
-	   )
+	if (   (f = open("/dev/isdninfo", O_RDONLY)) == -1
+		&& (f = open("/dev/isdn/isdninfo", O_RDONLY)) == -1
+	   ) 
 		{
 		if (_GK.debug_level & DEBUG_NET)
 			printf("sys_net_isdn__online: no /dev/isdninfo?\n");
 		return FALSE;
 		}
-	 for (i = 0; i < 5; i++)
+	memset(buffer, 0, BUFSIZ);
+
+	if (read(f, buffer, BUFSIZ) <= 0)
 		{
-		if (fgets (buffer, BUFSIZ, f) == NULL)
-			{
-			fclose (f);
-			return FALSE;
-			}
+		close(f);
+		return FALSE;
 		}
-	fclose (f);
-	if (strncmp (buffer, "flags:", 6))
+	close(f);
+
+	if ((p = strstr(buffer, "flags:")) == NULL)
 		return FALSE;
 
-	p = buffer+6;
-	while (*p)
+	for(p += 6; *p; )
 		{
-		if (isspace (*p))
+		if (isspace(*p))
 			{
 			p++;
 			continue;
 			}
-		for (end = p; *end && !isspace(*end); end++)
+		for	(end = p; *end && !isspace(*end); end++)
 			;
-		if (*end == '\0')
+		if (*end == '\0' || *end == '\t')
 			break;
 		else
 			*end = 0;
-		if (!strcmp (p, "?") || !strcmp (p, "0"))
+		if (!strcmp(p, "?") || !strcmp(p, "0"))
 			{
 			p = end+1;
 			continue;
@@ -1250,6 +1248,20 @@ static GList	*acpi_battery_list;
 #define	F_OK	0
 #endif
 
+static gchar *
+get_acpi_battery_file(gchar *dir, gchar *subdir, gchar *name)
+	{
+	gchar 	*path;
+
+	/* dir is expected to have trailing '/'
+	*/
+	path = g_strconcat(dir, subdir, "/", name, NULL);
+	if (!access(path, F_OK))
+		return path;
+	g_free(path);
+	return NULL;
+	}
+
 static gboolean
 setup_acpi_battery(gchar *bat)
 	{
@@ -1263,7 +1275,9 @@ setup_acpi_battery(gchar *bat)
 		bf = g_new0(BatteryFile, 1);
 		bf->id = id++;
 		bf->info = info;
-		bf->state = g_strconcat(ACPI_BATTERY_DIR, bat, "/state", NULL);
+		bf->state = get_acpi_battery_file(ACPI_BATTERY_DIR, bat, "state");
+		if (!bf->state)
+			bf->state = get_acpi_battery_file(ACPI_BATTERY_DIR, bat, "status");
 		acpi_battery_list = g_list_append(acpi_battery_list, bf);
 		return TRUE;
 		}
@@ -1276,14 +1290,12 @@ setup_ac_adapter(gchar **state, gchar *ac)
 	{
 	gchar	*path;
 
-	path = g_strconcat(ACPI_AC_ADAPTOR_DIR, ac, "/state", NULL);
-	if (!access(path, F_OK))
-		{
-		*state = path;
-		return TRUE;
-		}
-	g_free(path);
-	return FALSE;
+	path = get_acpi_battery_file(ACPI_AC_ADAPTOR_DIR, ac, "state");
+	if (!path)
+		path = get_acpi_battery_file(ACPI_AC_ADAPTOR_DIR, ac, "status");
+	*state = path;
+
+	return path ? TRUE : FALSE;
 	}
 
 static void
@@ -1321,12 +1333,30 @@ acpi_setup(void)
 		}
 	}
 
+static gboolean
+fgets_lower_case(gchar *buf, gint len, FILE *f)
+	{
+	guchar	*s;
+
+	if (!fgets(buf, len, f))
+		return FALSE;
+	s = (guchar *) buf;
+	if (isupper(*s))
+		while (*s)
+			{
+			if (isupper(*s))
+				*s = tolower(*s);
+			++s;
+			}
+	return TRUE;
+	}
+
 static gboolean
 acpi_battery_data(BatteryFile *bf)
 	{
 	FILE			*f;
 	gchar			buf[128], s1[32];
-	gboolean		on_line, charging, available;
+	gboolean		on_line, charging, available, have_charging_state;
 	gint			percent = 0, time_left = -1;
 	gint			cur_cap = 0, cur_rate = 0;
 	extern gint		gkrellm_battery_full_cap_fallback(void);
@@ -1336,7 +1366,7 @@ acpi_battery_data(BatteryFile *bf)
 		if ((f = fopen(bf->info, "r")) == NULL)
 			return FALSE;
 		bf->got_info = TRUE;
-		while(fgets( buf, sizeof(buf), f))
+		while (fgets_lower_case(buf, sizeof(buf), f))
 			{
 			/* present:					{yes|no}
 			|  design capacity:			53280 mWh
@@ -1366,11 +1396,13 @@ acpi_battery_data(BatteryFile *bf)
 		&& (f = fopen(acpi_ac_state_file, "r")) != NULL
 	   )
 		{
-		while (fgets(buf, sizeof(buf), f))
+		while (fgets_lower_case(buf, sizeof(buf), f))
 			{
 			/*	state: {on-line|off-line}
 			*/
-			if (   sscanf (buf, "state: %s", s1) == 1
+			if (   (   sscanf (buf, "state: %s", s1) == 1
+					|| sscanf (buf, "status: %s", s1) == 1
+				   )
 				&& !strcmp(s1, "on-line")
 			   )
 				on_line = TRUE;
@@ -1382,7 +1414,8 @@ acpi_battery_data(BatteryFile *bf)
 		return FALSE;
 	charging = FALSE;
 	available = FALSE;
-	while (fgets(buf, sizeof(buf), f))
+	have_charging_state = FALSE;
+	while (fgets_lower_case(buf, sizeof(buf), f))
 		{
 		/*	present:			{yes|no}
 		|	capacity state:		ok
@@ -1393,6 +1426,7 @@ acpi_battery_data(BatteryFile *bf)
 		*/
 		if (sscanf(buf, "charging state: %s", s1) == 1)
 			{
+			have_charging_state = TRUE;
 			if (   (!strcmp(s1, "unknown") && on_line)
 				|| !strcmp(s1, "charging")
 			   )
@@ -1413,6 +1447,9 @@ acpi_battery_data(BatteryFile *bf)
 	if (!acpi_ac_state_file && charging)	/* Assumption for buggy ACPI */
 		on_line = TRUE;
 
+	if (!have_charging_state && on_line)	/* Another buggy ACPI */
+		charging = TRUE;
+
 	percent = cur_cap * 100 / bf->full_cap;
 	if (percent > 100)
 		{
