@@ -44,6 +44,7 @@
 #include <PDHMSG.H>
 #include <LMCONS.H>
 #include <lmwksta.h>
+#include <lmapibuf.h>
 #else
 // windows 95/98/Me
 #include <tlhelp32.h>
@@ -75,6 +76,10 @@ static gint numAdapters = 0;
 static int rx[MAX_NET_ADAPTERS], tx[MAX_NET_ADAPTERS];
 static int initDone = 0;
 static OSVERSIONINFO info;
+static int fswin95 = 0;
+static gchar* sname;
+static gchar* hostname;
+
 #if defined(WIN32_CLIENT)
 static win32_plugin_callbacks plugin_callbacks;
 win32_plugin_callbacks* callbacks;
@@ -196,6 +201,8 @@ void gkrellm_sys_main_init(void)
 {
     WSADATA wsdata;
     int err;
+    HMODULE mod = 0;
+    FARPROC proc = 0;
 
     // initialize winsock
     if (_GK.debug_level & DEBUG_SYSDEP)
@@ -220,6 +227,17 @@ void gkrellm_sys_main_init(void)
     info.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
     GetVersionEx(&info);
 
+    // see if we can use the GetDiskFreeSpaceEx function
+    if (info.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS && info.dwMinorVersion == 0) {
+        // The GetDiskFreeSpaceEx function is available on Windows 95 systems beginning with OEM Service Release 2 (OSR2).
+        mod = LoadLibrary("KERNEL32.DLL");
+        proc = GetProcAddress(mod, "GetDiskFreeSpaceEx");
+        if (proc != NULL) {
+            fswin95 = 1;
+        }
+        FreeLibrary(mod);
+    }
+
     // we don't have local mail on Windows (yet?)
     gkrellm_mail_local_unsupported();
     
@@ -303,6 +321,9 @@ void gkrellm_sys_main_cleanup(void)
     for (i = 0; i < numPerfKeys; i++)
         g_free(perfKeys[i]);
 
+    g_free(sname);
+    g_free(hostname);
+
     // stop winsock
     if (_GK.debug_level & DEBUG_SYSDEP)
         printf("Closing Winsock\n");
@@ -560,11 +581,12 @@ gkrellm_sys_sensors_init(void)
 {
     int haveMBM = 0;
     char buf[25];
-    SharedData *ptr;
-    SharedSensor *sens;
-    HANDLE hSData;
-    int i, j;
-    int totalCount;
+//    SharedData *ptr;
+//    SharedSensor *sens;
+//    HANDLE hSData;
+    int i;
+//    int j;
+//    int totalCount;
 
     if (_GK.debug_level & DEBUG_SYSDEP)
         printf("Initializing sensors.\n");
@@ -584,7 +606,7 @@ gkrellm_sys_sensors_init(void)
 //        haveMBM = 0;
 //    }
 
-    if (haveMBM == 1) {
+/*    if (haveMBM == 1) {
         totalCount = 0;
         for (i = 0; i < 5; i++) {
           totalCount += ptr->sdIndex[i].Count;
@@ -632,7 +654,8 @@ gkrellm_sys_sensors_init(void)
         UnmapViewOfFile(ptr);
         CloseHandle(hSData);
     }
-    else {   
+    else  */
+    {   
         tempCount = 0;
         voltCount = 0;
         fanCount = 0;
@@ -805,11 +828,12 @@ gkrellm_sys_cpu_init(void)
             cpuperfKey = 0;
         }
         else {
-            if (RegOpenKey(HKEY_DYN_DATA, "PerfStats\\StatData", &cpuperfKey) != ERROR_SUCCESS)
+            if (RegOpenKey(HKEY_DYN_DATA, "PerfStats\\StatData", &cpuperfKey) != ERROR_SUCCESS) {
                 // failed
                 if (_GK.debug_level & DEBUG_SYSDEP)
                     printf("Initializing cpu failed.\n");
                 cpuperfKey = 0;
+            }
         }
         RegCloseKey(startPerf);
     }
@@ -934,9 +958,9 @@ gkrellm_sys_disk_init(void)
 #if defined(USE_PDH)
     DWORD size = 0;
     DWORD isize = 0;
-    gchar* objects;
-    gchar* instances;
-    gchar* instance;
+    char* objects;
+    char* instances;
+    char* instance;
     char buf[1024];
     int strSize;
     int i;
@@ -953,8 +977,8 @@ gkrellm_sys_disk_init(void)
             ++isize;
             ++size;
 
-            objects = (gchar*) malloc(sizeof(gchar) * size);
-            instances = (gchar*) malloc(sizeof(gchar) * isize);
+            objects = (char*) malloc(sizeof(char) * size);
+            instances = (char*) malloc(sizeof(char) * isize);
 
             PdhEnumObjectItems(NULL, NULL, TEXT("PhysicalDisk"), objects, &size, instances, &isize, PERF_DETAIL_WIZARD, 0);
 
@@ -1017,11 +1041,12 @@ gkrellm_sys_disk_init(void)
             diskreadPerfKey = 0;
         }
         else {
-            if (RegOpenKey(HKEY_DYN_DATA, "PerfStats\\StatData", &diskreadPerfKey) != ERROR_SUCCESS)
+            if (RegOpenKey(HKEY_DYN_DATA, "PerfStats\\StatData", &diskreadPerfKey) != ERROR_SUCCESS) {
                 // failed
                 if (_GK.debug_level & DEBUG_SYSDEP)
                     printf("Initialization failed.\n");
                 diskreadPerfKey = 0;
+            }
         }
         RegCloseKey(startPerf);
     }
@@ -1113,9 +1138,9 @@ gkrellm_sys_proc_read_data(void)
     HANDLE hSnap;
 #endif
     gint	n_running = 0, n_processes = 0;
-	double	n_forks = 0;
-    static double last_n_forks = 0;
-    double new_forks;
+	gulong	n_forks = 0;
+    static gulong last_n_forks = 0;
+    gulong new_forks;
 	static gfloat	fload = 0;
     gfloat a;
 
@@ -1192,14 +1217,15 @@ gkrellm_sys_proc_read_users(void)
 
 #if defined(USE_PDH)
     DWORD t1, t2;
-    LPWKSTA_USER_INFO_0 ptr;
+    LPWKSTA_USER_INFO_0 ptr = 0;
     NET_API_STATUS err;
 
-    err = NetWkstaUserEnum(NULL, 0, &ptr, 1024, &t1, &t2, NULL);
+    err = NetWkstaUserEnum(NULL, 0, (LPBYTE*) &ptr, 1024, &t1, &t2, NULL);
     if (err != 0 && err != ERROR_MORE_DATA)
     	n_users = 1;
     else
         n_users = t2;
+    NetApiBufferFree(ptr);
 #else
         n_users = 1;
 #endif
@@ -1237,11 +1263,12 @@ gboolean gkrellm_sys_proc_init(void)
             threadPerfKey = 0;
         }
         else {
-            if (RegOpenKey(HKEY_DYN_DATA, "PerfStats\\StatData", &threadPerfKey) != ERROR_SUCCESS)
+            if (RegOpenKey(HKEY_DYN_DATA, "PerfStats\\StatData", &threadPerfKey) != ERROR_SUCCESS) {
                 // failed
                 if (_GK.debug_level & DEBUG_SYSDEP)
                     printf("Initialization failed.\n");
                 threadPerfKey = 0;
+            }
         }
         RegCloseKey(startPerf);
     }
@@ -1411,6 +1438,9 @@ gkrellm_sys_net_init(void)
 #if defined(USE_PDH)
     if (winVer > 0 && query != 0) {
         PdhEnumObjectItems(NULL, NULL, TEXT("Network Interface"), NULL, &size, NULL, &isize, PERF_DETAIL_WIZARD, 0);
+        if (_GK.debug_level & DEBUG_SYSDEP) {
+            printf("Found %i network objects and %i network instances.\n", size, isize);
+        }
         if (size > 0) {
             ++isize;
             ++size;
@@ -1419,11 +1449,17 @@ gkrellm_sys_net_init(void)
             instances = (gchar*) malloc(sizeof(gchar) * isize);
 
             PdhEnumObjectItems(NULL, NULL, TEXT("Network Interface"), objects, &size, instances, &isize, PERF_DETAIL_WIZARD, 0);
+            if (_GK.debug_level & DEBUG_SYSDEP) {
+                printf("Enumerated %i network objects and %i network instances.\n", size, isize);
+            }
 
             for (instance = instances; *instance != 0; instance += lstrlen(instance) + 1) {
                 ++adapter;
-                if (adapter >= MAX_NET_ADAPTERS)
+                if (adapter >= MAX_NET_ADAPTERS) {
+                    if (_GK.debug_level & DEBUG_SYSDEP)
+                        printf("Hit maximum number of network adapters.\n");
                     break;
+                }
 
                 strSize = MAX_NET_NAME;
                 if (strlen(instance) < MAX_NET_NAME) {
@@ -1721,10 +1757,7 @@ void gkrellm_sys_fs_get_fsusage(gpointer fs, gchar *dir)
     ULARGE_INTEGER totalBytes;
     ULARGE_INTEGER freeBytes;
     gulong total, freeCaller, free;
-    HMODULE mod = 0;
-    FARPROC proc = 0;
     gulong secPerCluster, bytesPerSector, freeClusters, totalClusters;
-    int win95 = 0;
     HANDLE hFile;
     char buf[1024];
 
@@ -1742,14 +1775,7 @@ void gkrellm_sys_fs_get_fsusage(gpointer fs, gchar *dir)
         CloseHandle(hFile);
     }
 
-    if (info.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS && info.dwMinorVersion == 0) {
-        // The GetDiskFreeSpaceEx function is available on Windows 95 systems beginning with OEM Service Release 2 (OSR2).
-
-        mod = LoadLibrary("KERNEL32.DLL");
-        proc = GetProcAddress(mod, "GetDiskFreeSpaceEx");
-        win95 = 1;
-    }
-    if (proc != NULL || win95 == 0) {
+    if (fswin95 == 1) {
         err = GetDiskFreeSpaceEx(dir, &freeAvailableToCaller, &totalBytes, &freeBytes);
 
         if (err != 0) {
@@ -1761,7 +1787,7 @@ void gkrellm_sys_fs_get_fsusage(gpointer fs, gchar *dir)
 	        gkrellm_fs_assign_fsusage_data(fs, total, freeCaller, free, 1024);
         }
     }
-    if ((proc == NULL || err == 0) && win95 == 1) {
+    if (err == 0 && fswin95 == 0) {
         err = GetDiskFreeSpace(dir, &secPerCluster, &bytesPerSector, &freeClusters, &totalClusters);
         if (err != 0) {
             gkrellm_fs_assign_fsusage_data(fs, totalClusters * secPerCluster, freeClusters * secPerCluster, freeClusters * secPerCluster, bytesPerSector);
@@ -1779,8 +1805,6 @@ void gkrellm_sys_fs_get_mounts_list(void)
     for (drive = buf; *drive != 0; drive += lstrlen(drive) + 1) {
         // dir, dev, type
         if (strcmp("A:\\", drive) != 0 && strcmp("a:\\", drive) != 0 && strcmp("B:\\", drive) != 0 && strcmp("b:\\", drive) != 0) {
-            if (_GK.debug_level & DEBUG_SYSDEP)
-                printf("Adding mount %s\n", drive);
     	    gkrellm_fs_add_to_mounts_list(drive, drive, "");
         }
     }    
@@ -1900,15 +1924,18 @@ gkrellm_sys_uptime_init(void)
 gchar *
 gkrellm_sys_get_host_name(void)
 	{
-	static gboolean	have_it;
-	static gchar	buf[128];
+	static gboolean	have_it = FALSE;
+    char buf2[128];
     int err;
 
     if (!have_it) {
-        err = gethostname(buf, sizeof(buf));
+        err = gethostname(buf2, 128);
 
         if (err != 0) {
-            strcpy(buf, "Unknown");
+            hostname = g_strdup("Unknown");
+        }
+        else {
+            hostname = g_strdup(buf2);
         }
 
         if (_GK.debug_level & DEBUG_SYSDEP)
@@ -1917,7 +1944,7 @@ gkrellm_sys_get_host_name(void)
         have_it = TRUE;
     }
 
-	return buf;
+	return hostname;
 	}
 
 /* ===================================================================== */
@@ -1927,15 +1954,14 @@ gkrellm_sys_get_host_name(void)
 gchar *
 gkrellm_sys_get_system_name(void)
 	{
-	static gchar	*sname;
+	static gboolean	have_it = FALSE;
     char osName[255];
     char build[255];
 
     if (_GK.debug_level & DEBUG_SYSDEP)
         printf("Retrieving system name.\n");
 
-    if (!sname) {
-
+    if (!have_it) {
         strcpy(osName, "Unknown");
         strcpy(build, "");
 
@@ -1974,7 +2000,9 @@ gkrellm_sys_get_system_name(void)
             }
         }
 
-    	sname = g_strdup_printf("%s %s", osName, "");
+        sname = g_strdup(osName);
+        have_it = TRUE;
+    	//sname = g_strdup_printf("%s %s", osName, "");
     }
 
 
