@@ -452,6 +452,26 @@ typedef struct {
     GkrellmDecal *(*gkrellm_make_scaled_decal_pixmap)(GkrellmPanel *p,
                         GkrellmPiximage *im, GkrellmStyle *style, gint depth,
                         gint x, gint y, gint w, gint h);
+
+      /* Functions new for 2.1.8
+      */
+#if 0
+	void		(*gkrellm_panel_label_on_top_of_decals)(GkrellmPanel *p,
+						gboolean mode);
+	gboolean	(*gkrellm_alert_is_activated)(GkrellmAlert *alert);
+	void		(*gkrellm_alert_dup)(GkrellmAlert **ap, GkrellmAlert *a_src);
+	void		(*gkrellm_alert_config_create_connect)(GkrellmAlert *alert,
+						void (*func)(), gpointer data);
+	void		(*gkrellm_alert_command_process_connect)(GkrellmAlert *alert,
+						void (*func)(), gpointer data);
+	gboolean	(*gkrellm_alert_decal_visible)(GkrellmAlert *alert);
+	void		(*gkrellm_alert_set_delay)(GkrellmAlert *alert, gint delay);
+	void		(*gkrellm_alert_delay_config)(GkrellmAlert *alert, gint step,
+						gint high, gint low);
+	void		(*gkrellm_gtk_alert_button)(GtkWidget *box, GtkWidget **button,
+						gboolean expand, gboolean fill, gint pad,
+						gboolean pack_start, void (*cb_func)(), gpointer data);
+#endif
  
 
 } win32_plugin_callbacks;
