@@ -215,11 +215,18 @@ static FILE		*f_lock;
 static gboolean
 _gkrellm_get_lock(void)
 	{
-	gchar	*lock_file;
+	gchar	*lock_file, *display, *s;
 
 #if defined(F_TLOCK)
 	lock_file = gkrellm_make_config_file_name(gkrellm_homedir(),
 					GKRELLM_LOCK_FILE);
+	display = XDisplayName(NULL);
+	if (display)
+		{
+		s = g_strconcat(lock_file, "_", display, NULL);
+		g_free(lock_file);
+		lock_file = s;
+		}
 	f_lock = fopen(lock_file, "w+");	/* buffering does not apply here */
 	g_free(lock_file);
 	if (!f_lock || lockf(fileno(f_lock), F_TLOCK, 0) != 0)
@@ -283,13 +290,19 @@ gkrellm_winop_options(gint argc, gchar **argv)
 					XInternAtom(display, "_NET_WM_STATE", False),
 					XA_ATOM, 32, PropModeReplace, (guchar *) atoms, n);
 
-	if (_GK.withdrawn && !_GK.is_dock_type)
+	if (_GK.withdrawn)
 		{
-		XWMHints mywmhints; 
-		mywmhints.initial_state = WithdrawnState; 
-		mywmhints.flags=StateHint;
+		if (!_GK.is_dock_type)
+			{
+			XWMHints mywmhints; 
+			mywmhints.initial_state = WithdrawnState; 
+			mywmhints.flags=StateHint;
 
-		XSetWMHints(display, window, &mywmhints); 
+			XSetWMHints(display, window, &mywmhints); 
+			}
+		else
+			gkrellm_message_dialog(NULL,
+			_("Warning: -w flag is ignored when the window dock type is set"));
 		}
 	} 
 
